package sim;

import java.util.ArrayList;
import java.util.List;

public class Channel
{
	private List notifylist = new ArrayList();
	private List gets_notify_from = new ArrayList();
	
	public static final int TYPE_INOUT = 1;
	public static final int TYPE_IN = 2;
	public static final int TYPE_OUT = 3;
	
	private String name;
	
	private final int type;
	private final Entity e;
	
	protected Object state;
	protected Object data;
	private Channel rchannel;
	
	private boolean active = false;
	private boolean sending = false;
	private boolean init_state = false;

	public Channel(Entity e)
	{
		e.regChannel(this);
		this.e = e;
		this.type = TYPE_INOUT;		
	}
	
	public void setName(String name)
	{
		this.name = name;
	}
	
	public String getName()
	{
		getEntity().initChannelNames();		
		return name;
	}
	
	public int getType()
	{
		return type;
	}
	
	public boolean isActive()
	{
		return active;
	}
	
	protected boolean checkData(Object o, Object b)
	{
		return true;
	}
	
	private boolean _checkData(Object o, Object b)
	{		
		if((b == null && o != null) || (o == null && b != null))
			return true;
		else 
			return checkData(o,b);
	}
	
	public void writeObject(Object o, int i)
	{		
		data = o;
		
		sending = true;
		Channel ch = (Channel) notifylist.get(i);
		ch.reciveObject(this);
		sending = false;
	}
	
	public void writeObject(Object o)
	{
		if(o instanceof Channel)
			throw new RuntimeException("Kan ikke sende objekt av type: Channel");
		
		data = o;
		
		for(int i = 0; i < notifylist.size(); i++)
			writeObject(o, i);
	}
	
	protected void reciveObject(Channel c)
	{
		if(sending || !_checkData(c.data,data) || !c._checkData(c.data,data))
			return;
		
		data = c.data;
		rchannel = c;
		active = true;
		
		writeObject(data);
	}
	
	public Entity getEntity()
	{
		return e;
	}
	
	public Object readObject()
	{		
		active = false;
		return data;
	}
	
	public Object[] readObjects()
	{		
		Object[] list = new Object[gets_notify_from.size()];
		for(int i = 0; i < gets_notify_from.size(); i++)
		{
			Channel ch = (Channel) gets_notify_from.get(i);
			list[i] = ch.data;
		}
		return list;
	}
	
	public void initStateObject(Object o)
	{
		init_state = true;
		
		writeObject(o);
		commit();
	}
	
	public Object getStateObject()
	{
		return state;
	}
	
	public void commit()
	{
		state = data;
	}	
	
	public void connect(Channel ch)
	{
		ch.notifylist.add(this);
		gets_notify_from.add(ch);
		
		if(init_state)
			return;
		
		data = ch.data;
		state = ch.state;
	}
	
	
	public void connect(Entity ent)
	{
		if(ent.getChannel("out") == null)
			throw new RuntimeException("kan ikke finne ut for: " +ent.getName());
		
		connect(ent.getChannel("out"));
	}
	
	public List getConnectedChannels()
	{
		return gets_notify_from;
		//return connected_to_list;
	}
	
	public boolean isFloating()
	{
		if(gets_notify_from.isEmpty() && notifylist.isEmpty())
			return true;
		
		return false;
	}
	
	public String trace()
	{
		String ret = "";
		
		ret += getEntity().getName() + "." + getName();
		if(rchannel != null)
			ret += " <- " +rchannel.getEntity().getName() + "." + rchannel.getName();
		else
			ret += " <- null";
		
		ret += " [" + data + "]";
		return ret;
	}
	
	public String toString()
	{
		return getEntity().getName() + "." + getName() + " [" + data + "]";
	}
}
