package sim;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

public abstract class Entity
{
	private final Sim sim;
	private List channels = new ArrayList();
	private List commits = new ArrayList();
	
	private Channel achannel;
	
	private String name;
	private boolean init = false;

	public Entity(Sim sim)
	{
		this.sim = sim;
		sim.regEntity(this);
	}
	
	public Entity()
	{
		this(Sim.getCurrentSim());
	}
	
	public Entity connect(Entity e)
	{
		if(getChannel("in") != null && e.getChannel("out") != null)
		{
			getChannel("in").connect(e.getChannel("out"));
			return this;
		}
		
		for(int i = 0; i < e.channels.size(); i++)
		{
			Channel too = (Channel) e.channels.get(i);
			if(_connect(too))
				break;
		}
		
		return this;
	}
	
	boolean _connect(Channel too)
	{
		int itoo = too.getName().indexOf('_');
		if(itoo == -1)
			return false;
		
		String stoo = too.getName().substring(itoo,too.getName().length());			
		
		for(int j = 0; j < channels.size(); j++)
		{
			Channel tis = (Channel) channels.get(j);
			
			int itis = tis.getName().indexOf('_');
			if(itis == -1)
				continue;
			
			String prefix = tis.getName().substring(0,itis);
			//System.out.println(prefix);
			if(!(prefix.equals("in") || prefix.equals("io")))
				continue;
			
			String stis = tis.getName().substring(itis,tis.getName().length());
			
			if(stoo.equals(stis))
			{
				tis.connect(too);
				//System.out.println("connecting: " + tis + " -> " + too);
				return true;
			}
		}
		
		return false;		
	}
	
	public Entity connect(Channel c)
	{
		connectTest(c);
		return this;
	}
	
	public boolean connectTest(Channel c)
	{
		if(getChannel("in") != null)
		{
			getChannel("in").connect(c);
			return true;
		}
		
		return _connect(c);		
		//return this;
	}
	
	public void regCommit(ICommit commit)
	{
		commits.add(commit);
	}
	
	public String getName()
	{
		initChannelNames();
		return name;
	}
	
	public List getChannels()
	{
		return channels;
	}
	
	public Channel getChannel(String ch_name)
	{
		initChannelNames();
		
		for(int i = 0; i < channels.size(); i++)
		{
			Channel c = (Channel) channels.get(i);
			String cname = c.getName();
			if(cname != null && cname.equals(ch_name))
				return c;
		}
		return null;
	}
	
	void initChannelNames()
	{
		if(init)
			return;
		
		init = true;
		
		name = getClass().getSimpleName();
		Field[] fields = getClass().getDeclaredFields();
		for(int i = 0; i < fields.length; i++)
		{
			Field f = fields[i];
			f.setAccessible(true);
			if(!Channel.class.isAssignableFrom(f.getType()))
				continue;
			try
			{
				Object o = f.get(this);
				if(o == null)
					continue;
				
				for(int j = 0; j < channels.size(); j++)
				{
					Channel c = (Channel) channels.get(j);
					if(o.equals(c))
					{
						c.setName(f.getName());
						break;
					}
				}
				
			} catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
		
		
	}
	
	public void setSensitive(Channel c)
	{
		achannel = c;
	}
	
	public boolean isActive()
	{
		if(achannel != null)
			return achannel.isActive();
		
		for(int i = 0; i < channels.size(); i++)
		{
			Channel c = (Channel) channels.get(i);
			if(c.isActive())
				return true;
		}
		return false;
	}
	
	void resetActive()
	{
		for(int i = 0; i < channels.size(); i++)
		{
			Channel c = (Channel) channels.get(i);
			c.readObject();
		}
	}
	
	void printActive()
	{
		boolean init = false;
		for(int i = 0; i < channels.size(); i++)
		{
			Channel c = (Channel) channels.get(i);
			if(c.isActive())
			{
				if(init)
					System.out.print(" ");
				
				System.out.println(c.trace());
				init = true;
			}
		}
	}
	
	protected void commit()
	{
		for(int i = 0; i < channels.size(); i++)
		{
			Channel c = (Channel) channels.get(i);
			c.commit();
		}
		
		for(int i = 0; i < commits.size(); i++)
		{
			ICommit c = (ICommit) commits.get(i);
			c.commit();
		}
	}
	
	void regChannel(Channel c)
	{
		channels.add(c);
	}
	
	public Sim getSim()
	{
		return sim;
	}
	
	protected void reset()
	{
		
	}
	
	protected void read()
	{
		
	}
	
	protected void write()
	{
		
	}
	
	protected void tick()
	{
		
	}
	
	protected void post_tick()
	{
		
	}
	
	protected void rollback()
	{
		for(int i = 0; i < commits.size(); i++)
		{
			ICommit c = (ICommit) commits.get(i);
			c.rollback();
		}
	}
	
	
}
