package sim;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Sim
{
	private static HashMap thread_sim_map = new HashMap();
	
	private List readers = new ArrayList();
	private List entities = new ArrayList();
	private List solvers = new ArrayList();
	
	private List auto = new ArrayList();
	
	private boolean debug = false;
	private boolean _debug = false;
	private int debug_level = 1;
	private int debug_mod = 1;
	
	private int num_runs = 0;

	private boolean go;
	private boolean init = false;
	
	public Sim()
	{
		thread_sim_map.put(Thread.currentThread(), this);
	}
	
	public void autoConnect(Channel c)
	{
		auto.add(c);
	}
	
	public static Sim getCurrentSim()
	{
		return (Sim) thread_sim_map.get(Thread.currentThread());
	}
	
	public void setDebug(boolean debug)
	{
		this.debug = debug;
		_debug = debug;
	}
	
	public void setDebugLevel(int level)
	{
		this.debug_level = level;
	}
	
	public void setDebugMod(int num)
	{
		debug_mod = num;
	}
	
	public boolean isDebugOn()
	{
		return _debug;
	}
	
	public int getDebugLevel()
	{
		return debug_level;
	}
	
	public void regSampler(ISample r)
	{
		readers.add(r);
	}
	
	public void regEntity(Entity e)
	{
		entities.add(e);
	}
	
	private void _autoConnect(Entity e)
	{
		for(int i = 0; i < auto.size(); i++)
		{
			Channel c = (Channel) auto.get(i);
			e._connect(c);

			
			/*if(e._connect(c))
			{
				//System.out.println("Connecting: " +e + " too " + c);
				break;
			}*/
		}		
	}
	
	private void _initAutoConnect()
	{
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			_autoConnect(e);
		}
	}
	
	public void regSolv(ISolver solver)
	{
		solvers.add(solver);
	}
	
	public void init()
	{
		if(init)
			return;
		
		init = true;
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.initChannelNames();
		}
		
		_initAutoConnect();
		
		if(debug)
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			List list = e.getChannels();
			for(int j = 0; j < list.size(); j++)
			{
				Channel c = (Channel) list.get(j);
				if(c.isFloating())
					System.out.println("Warning, unconnected: " + c);
			}
		}
	}
	
	public void stop()
	{
		go = false;
	}
	
	public void play(int max_it)
	{
		play(Integer.MAX_VALUE,max_it);
	}
	
	public void play()
	{
		play(20);
	}

	public void play(int num_steps, int max_it)
	{
		init();
		go = true;
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.reset();
		}
		
		for(int i = 0; i < num_steps && go; i++)
			_play(max_it);
		
		//Thread.yield();
	}
	
	private void _play(int max_it)
	{
		if(debug && num_runs % debug_mod == 0)
			_debug = true;
		else
			_debug = false;
		
		if(_debug)
			System.out.println("*");
		
		solvers.clear();
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.read();
		}
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.write();
		}
		
		for(int i = 0; i < readers.size(); i++)
		{
			ISample r = (ISample) readers.get(i);
			r.read();
		}
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.tick();
			e.resetActive();
		}
		
		boolean not_done = true;
		for(int i = 0; i < max_it && not_done; i++)
		{
			if(_debug && debug_level >= 4)
			{
				System.out.println("-");
				for(int j = 0; j < entities.size(); j++)
				{
					Entity e = (Entity) entities.get(j);
					if(e.isActive())
						e.printActive();		
				}
				
				System.out.println(">");
			}
			
			
			not_done = false;
			for(int j = 0; j < entities.size(); j++)
			{
				Entity e = (Entity) entities.get(j);
				if(e.isActive())
				{
					if(_debug && debug_level >= 3)
						e.printActive();
					
					e.rollback();
					e.tick();
					e.resetActive();
					not_done = true;
				}				
			}
			
			if(not_done == false && !solvers.isEmpty())
			{
				not_done = true;
				ISolver s = (ISolver) solvers.remove(solvers.size()-1);
				s.stableInputs();				
			}
			//System.out.println("-");
		}
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.post_tick();
		}
		
		for(int i = 0; i < entities.size(); i++)
		{
			Entity e = (Entity) entities.get(i);
			e.commit();
		}
		
		num_runs++;
	}
}
