package sim;

import sim.test2.IntegerChannel;
import team.app.div.ITime;

public class Time extends Entity implements ITime
{
	public DoubleChannel io_delta = new DoubleChannel(this);
	public DoubleChannel out_time = new DoubleChannel(this);
	public IntegerChannel out_day = new IntegerChannel(this);
	public DoubleChannel out_total_time = new DoubleChannel(this);
	
	private double day_in_s = 24*60*60;
	private double stop_time = Double.MAX_VALUE;
	
	
	public Time(double time, int day)
	{
		out_time.initState(time);
		io_delta.initState(0.1);
		out_day.initState(day);
		out_total_time.initState(out_day.read() * day_in_s + out_time.read());
	}
	
	public Time(double time)
	{
		this(time,0);
	}
	
	public Time()
	{
		this(0);
	}
	
	public float getTotalTime()
	{
		return (float)out_total_time.read();
	}
	
	public float getSimTime()
	{
		return (float)out_time.read();
	}

	public float getDeltaTime()
	{
		return (float)io_delta.read();
	}
	
	public void setStopTime(float time)
	{
		stop_time = time;
	}
	
	public static String format(double time)
	{
		boolean neg = false;
		if(time < 0)
		{
			neg = true;
			time = -time;
		}
			
		
		double tmp = time / 60 /60;
		int hr = (int) tmp;
		tmp = tmp - hr;
		tmp *= 60;
		int min = (int) tmp;
		tmp = tmp - min;
		tmp *= 60;
		int sek = (int)Math.round(tmp);
		
		StringBuilder ret = new StringBuilder();
		
		if(neg)
			ret.append("-");
		
		if(hr < 10)
			ret.append("0");
		ret.append(hr).append(":");
		if(min < 10)
			ret.append("0");
		ret.append(min).append(":");		
		if(sek < 10)
			ret.append("0");
		ret.append(sek);
		
		return ret.toString(); 
	}
	
	public static float toDouble(int hr, int min, int sek)
	{
		return hr*60*60 + min*60 + sek;
	}
	
	public static float toDouble(String tid)
	{
		//format: 0739
		tid = tid.trim();
		int hr = Integer.parseInt(tid.substring(0, 2));
		int min = Integer.parseInt(tid.substring(2, 4));
		return toDouble(hr, min, 0);
	}
	
	protected void read()
	{
		if(getSim().isDebugOn())
			System.out.println("Time: " + out_time.read());
		
		if(out_time.read() > stop_time)
			getSim().stop();
	}
	
	protected void tick()
	{
		io_delta.write(io_delta.read());
		out_time.write(out_time.getState() + io_delta.read());
		
		if(out_time.read() > day_in_s)
		{
			out_day.put(out_day.getState()+1);
			out_time.write(out_time.read() - day_in_s);
		}
		
		out_total_time.write(out_day.read() * day_in_s + out_time.read());
	}
}
