package team.app.delay;

import org.jscience.mathematics.numbers.Float64;
import org.jscience.mathematics.vectors.Matrix;

import team.app.delay.Plot.Plotter;
import team.app.div.Random;

public class Kalman2
{
	double meas_noise = 0.2;
	double a_noise = 0.1;
	
	Matrix H = Matrix.valueOf(new double[][] {{1,0}});		
	
	Matrix Xhat = Matrix.valueOf(new double[][] {{0},{0}});
	
	//Q prosessnoise
	//Matrix Tmp = Matrix.valueOf(new double[][] {{0.05},{0.14}});
	Matrix Tmp = Matrix.valueOf(new double[][] {{0.01},{0.15}});
	Matrix Q = Tmp.times(Tmp.transpose()).times(Float64.valueOf(1));
	Matrix P = Q;
	
	//R messurmencenoise
	Matrix R = Matrix.valueOf(new double[][] {{Math.pow(meas_noise,2)}});
	
	public Kalman2()
	{
		
	}
	
	public void set(double value, double speed)
	{
		Xhat = Matrix.valueOf(new double[][] {{value},{speed}});
	}
	
	public void reg(double value, double dt)
	{				
		//System.out.println(Q);
		
		Matrix A = Matrix.valueOf(new double[][] {{1,dt},{0,1}});		
		Matrix Z = H.times(Matrix.valueOf(new double[][] {{value},{0}}));
		
		Xhat = A.times(Xhat);
		P = A.times(P).times(A.transpose()).plus(Q);
		
		// Predict OK
		
		Matrix tmp = H.times(P).times(H.transpose()).plus(R);
		Matrix K = P.times(H.transpose()).times(tmp.inverse());
		Xhat = Xhat.plus(K.times(Z.minus(H.times(Xhat))));
		P = Matrix.valueOf(new double[][] {{1,0},{0,1}}).minus(K.times(H)).times(P);		
	}
	
	public double estimatePos()
	{
		return ((Float64)Xhat.get(0, 0)).doubleValue();
	}
	
	public double estimateSpeed()
	{
		return ((Float64)Xhat.get(1, 0)).doubleValue();
	}
	
	public static void main(String[] args) throws Exception
	{
		//Random.setTimeSeed();
		Random.setSeed(102);
		
		Plot p = new Plot();
		
		Plotter r = p.getPlotter("real pos");
		Plotter v = p.getPlotter("obs pos");
		Plotter e = p.getPlotter("est pos");
		Plotter s = p.getPlotter("est speed");	
		
		Kalman2 k = new Kalman2();
		k.set(0,0);
		
		for(int i = 0; i < 20; i++)
		{
			double delay = ((double)i) / 2 * 60;
			if(i >= 10)
				delay = 5 * 60;
			
			double obs_delay = Random.randn(delay,0.5*60);
			k.reg(obs_delay, 0.2); 
			
			v.plot(obs_delay);
			e.plot(k.estimatePos());
			s.plot(k.estimateSpeed());
			r.plot(delay);
			p.step(500);
		}
		
		p.setVisible(true);
	}
}
