package team.app.div;

import idb.dom.Node;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.jscience.mathematics.vectors.Matrix;

import team.gui.CoordTransform;
import team.gui.Coord;
import team.gui.IPaint;
import team.gui.InputGUI;
import team.gui.ShapePainter;

import nextgen.utils.uFile;

public class HldDrawer implements IPaint
{
	Node rute;
	
	Matrix utm_world = CoordTransform.getUTM2WorldTransform();
	CoordTransform c_utm_world = new CoordTransform(CoordTransform.getUTM2WorldTransform());
	
	private final ShapePainter painter;
	private long seed = 1145280220747l;
	private Random rnd = new Random();
	
	private List holdeplasser;
	
	public HldDrawer(ShapePainter painter) throws Exception
	{
		this.painter = painter;
		painter.registerPainter(this);
		rute = Node.parse(uFile.readString("rute.txt"));	
		initHoldeplasser();
		
		InputGUI.getBuilder("Map").addButton("Mapdata","Refresh").addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				try
				{
					rute = Node.parse(uFile.readString("rute.txt"));
					seed = System.currentTimeMillis();
					System.out.println("seed: " + seed);
				} catch (Exception e1)
				{
					e1.printStackTrace();
				}
			}});
	}
	
	private void paintTerms(Graphics2D g2d)
	{
		Node term = rute.select("terminaler");
		List list = term.nodes("term");
		for(int i = 0; i < list.size(); i++)
		{
			Node t = (Node) list.get(i);
			String name = t.nodeText("navn");
			String c = t.nodeText("wcoord");
			String[] cs = c.split(",");
			
			double x = Double.parseDouble(cs[0]);
			double y = Double.parseDouble(cs[1]);
			
			Color org = g2d.getColor();
			g2d.setColor(Color.RED);
			
			painter.setNoScale(g2d, x, y);
			g2d.translate(-5, -5);
			g2d.fillOval(0, 0, 10, 10);
			g2d.setColor(org);
			g2d.drawString(name, 0, 0);
			
		}
	}
	
	private void initHoldeplasser() throws Exception
	{
		holdeplasser = new ArrayList();
		Node hld = Node.parse(uFile.readString("holdeplasser.txt"));
		
		List list = hld.selectAll("hld");
		for(int i = 0; i < list.size(); i++)
		{
			Node t = (Node) list.get(i);
			String dir = t.nodeText("dir");
			
			String name = t.nodeText("name");
			String c = t.nodeText("utm");
			
			int index = c.indexOf(',');				    	
	    	double x = Double.parseDouble(c.substring(0, index));
	    	double y = Double.parseDouble(c.substring(index+1, c.length()));
			
			Coord wpos = c_utm_world.getWorldCoord(x, y);
			x = wpos.x;
			y = wpos.y;
			
			Hld h = new Hld();
			h.name = name;
			h.dir = dir;
			h.x = x;
			h.y = y;
			
			holdeplasser.add(h);			
		}
	}
	
	private void paintHoldeplasser(Graphics2D g2d)
	{ 
		double b[] = painter.getBoundryWCoords();
		for(int i = 0; i < holdeplasser.size(); i++)
		{
			Hld h = (Hld) holdeplasser.get(i);
			if(!h.dir.equals("P"))
				continue;
			
			if(h.x < b[0] || h.x > b[2] ||
					h.y < b[1] || h.y > b[3])
				continue;
			
			Color org = g2d.getColor();
			g2d.setColor(Color.RED);
			
			painter.setNoScale(g2d, h.x, h.y);
			g2d.translate(-5, -5);
			g2d.fillOval(0, 0, 10, 10);
			g2d.setColor(org);
			g2d.drawString(h.name, 0, 0);
			
		}
	}
	
	
	
	public void paintLanes(Graphics2D g2d)
	{
		rnd.setSeed(seed);
		
		Stroke orgs = g2d.getStroke();
		Stroke wide = new BasicStroke(10);
		
		List lanes = rute.selectAll("lanepoints/lane");
		for(int i = 0; i < lanes.size(); i++)
		{
			Node lain = (Node) lanes.get(i);
			String id = lain.nodeText("id");
			List coords = lain.selectAll("wcoord");
			
			Color org = g2d.getColor();
			g2d.setColor(new Color(rnd.nextFloat(),rnd.nextFloat(),rnd.nextFloat()));
			
			double ox = -1;
			double oy = -1;
			
			for(int j = 0; j < coords.size(); j++)
			{
				String c = ((Node)coords.get(j)).text();
				String[] cs = c.split(",");
				
				double x = Double.parseDouble(cs[0]);
				double y = Double.parseDouble(cs[1]);
				
				
				painter.reset(g2d);
				
				if(ox != -1)
				{
					g2d.setStroke(wide);
					g2d.drawLine((int)ox, (int)oy, (int)x, (int)y);
					g2d.setStroke(orgs);
				}
				
				if(j == 0)
				{
					Font f = g2d.getFont();
					Font nf = f.deriveFont(50f);
					g2d.setFont(nf);					
					g2d.drawString("Lane: " + id, (float)x+5, (float)y+5);
					g2d.setFont(f);
				}
				
				//view.setNoScale(g2d, x, y);
				
				g2d.translate(x+-5, y+-5);
				g2d.fillOval(0, 0, 10, 10);
				
				ox = x;
				oy = y;
			}
			
			g2d.setColor(org);
		}		
	}
	
	private static class Hld
	{
		String name;
		String dir;
		double x;
		double y;
	}

	public void paint(Graphics2D g2d)
	{
		//paintHoldeplasser(g2d);
		
		paintLanes(g2d);
		paintHoldeplasser(g2d);
	}
}
