package team.app.models;

import java.awt.Polygon;
import java.util.List;

import idb.dom.Node;
import sim.DoubleChannel;
import sim.Entity;
import sim.CoordChannel;
import sim.Time;
import team.app.StdConfig;
import team.app.system.ScheduleCalc;
import team.app.view.PolyPainter.IPoly;
import team.app.wp.WPUtils;
import team.gui.Coord;

public class Area extends Entity implements IPoly
{
	public DoubleChannel in_time = new DoubleChannel(this);
	public CoordChannel in_positions = new CoordChannel(this);
	public DoubleChannel out_trafic = new DoubleChannel(this);
	
	private Polygon poly;
	//private double trafic;
	private TrafikkFlyt trafikk;
	
	public Area(Node omr)
	{
		poly = new Polygon();
		List cs = omr.selectAll("wcoord");
		for(int i = 0; i < cs.size(); i++)
		{
			Node n = (Node) cs.get(i);
			Coord c = WPUtils.parseWCoord(n.text());
			poly.addPoint((int)c.x, (int)c.y);
		}
		
		trafikk = new TrafikkFlyt(omr.node("trafikkflyt"));
		
		//trafic = Double.parseDouble(omr.nodeText("trafikk"));
		
		//String t = omr.nodeText("trafikkflyt");
		//String[] ts = t.split(";");
		
	}
	
	public Polygon getPolygon()
	{
		return poly;
	}
	
	public void read()
	{
		if(getSim().isDebugOn())
			System.out.println(trafikk.getTrafic(in_time.read()));
	}
	
	public void tick()
	{
		out_trafic.write(1);
		Coord[] coords = in_positions.readAll();
		for(int i = 0; i < coords.length; i++)
		{
			Coord c = coords[i];
			if(poly.contains(c.x,c.y))
				out_trafic.write(trafikk.getTrafic(in_time.read()), i);
		}
	}
	
	private static class TrafikkFlyt
	{
		private final Node config;
		
		private double flyt;
		private double periode;
		private double std_avvik;
		
		private double[] tid = new double[100];
		private double[] verdi = new double[100];
		private int size = 0;
		
		public TrafikkFlyt(Node config)
		{
			this.config = config;
			flyt = Double.parseDouble(config.nodeText("flyt"));
			periode = Double.parseDouble(config.nodeText("periode"));
			std_avvik = Double.parseDouble(config.nodeText("std_avvik"));
			
			String[] m = config.nodeText("mnster").split(";");
			for(int i = 0; i < m.length; i+=2)
			{
				tid[size] = Time.toDouble(m[i]);
				verdi[size] = Double.parseDouble(m[i+1]);
				size++;				
			}
		}
		
		public double getTrafic(double timeofday)
		{
			for(int i = 1; i < size; i++)
			{
				if(tid[i]>timeofday)
					return ScheduleCalc.interpolate(verdi[i-1],verdi[i], ScheduleCalc.ratio(tid[i-1], tid[i], timeofday))  * flyt;
			}
			
			return flyt;
		}
	}
	
	public static void main(String[] args) throws Exception
	{
		for(int i = 0; i < 18; i++)
		{
			double t = Time.toDouble(i, 0, 0);
			double flyt = new TrafikkFlyt(new StdConfig().getConfig("rute.txt").select("omrder/omrde/trafikkflyt")).getTrafic(t);
			System.out.println("Flyt " + i +": "+ flyt/0.7);
		}
	}
}
