package team.app.models;

import org.jscience.mathematics.numbers.Float64;
import org.jscience.mathematics.numbers.Number;
import org.jscience.mathematics.structures.Field;
import org.jscience.mathematics.vectors.Vector;

import sim.DoubleChannel;
import sim.Entity;
import sim.Sim;
import team.app.IConfig;
import team.app.StdConfig;
import team.app.wp.WPUtils;
import team.app.wp.WayPoints;
import team.gui.Coord;

public class RouteCurves extends Entity
{	
	public DoubleChannel in_dist_route_end = new DoubleChannel(this);
	
	public DoubleChannel out_length_to_curve = new DoubleChannel(this);
	public DoubleChannel out_curve_speed = new DoubleChannel(this);
	public DoubleChannel out_length_to_curve2 = new DoubleChannel(this);
	public DoubleChannel out_curve_speed2 = new DoubleChannel(this);
	
	private double scale = 1;
	
	private double[] length;
	private double[] speed;

	private final Schedule schedule;

	private boolean tmp_next_rout;

	private final IConfig config;

	public RouteCurves(IConfig config, Schedule schedule)
	{
		this.config = config;
		this.schedule = schedule;			
	}
	
	private static Field length(Vector v)
	{
		double ret = 0;
		for(int i = 0; i < v.getDimension(); i++)
		{
			Number n = (Number)v.get(i);
			ret += n.doubleValue() * n.doubleValue();
			
		}
		return Float64.valueOf(Math.sqrt(ret));
	}
	
	public void setWPS(WayPoints wps)
	{
		Coord c1 = wps.getWP(0);
		Coord c2 = wps.getWP(1);
		
		length = new double[wps.size()-3];
		speed = new double[wps.size()-3];
		
		double tlength = WPUtils.getTotalLength(wps);
		double alength = c2.distance(c1);
		
		for(int i = 2; i < wps.size()-1; i++)
		{
			Coord c3 = wps.getWP(i);
			
			Vector v1 = c1.toVector();
			Vector v2 = c2.toVector();
			Vector v3 = c3.toVector();
			
			v1 = v1.minus(v2);
			v3 = v3.minus(v2);			
			
			
			v1 = v1.times((Field)length(v1).inverse());
			v3 = v3.times((Field)length(v3).inverse());
			
			v1 = v1.opposite();
			
			//System.out.print((int)alength);
			
			//v1.times(v3)
			
			Number ang = (Number) v1.times(v3);
			double deg = Math.acos(ang.doubleValue())/3.14*180;
			
			//System.out.print(" Ang: " + deg);
			
			length[i-2] = tlength - alength;
			
			if(deg < 60)
				speed[i-2] = scale*(1 - (deg / 60)*0.5);
			else
				speed[i-2] = scale*0.05;
			
			//System.out.println(" speed: " + speed[i-1]);
			
			alength += c3.distance(c2);
			c1 = c2;
			c2 = c3;
		}		
	}
	
	public double getNextSpeed()
	{
		return out_curve_speed.read();
	}
	
	public double getLength()
	{
		return out_length_to_curve.read();
	}
	
	public void reset()
	{
		setWPS(WPUtils.loadRoute(schedule.getRouteID(), config));
	}
	
	public void read()
	{
		tmp_next_rout = false;
		if(schedule.eventNextRoute())
			tmp_next_rout = true;
	}
	
	public void write()
	{
		if(tmp_next_rout)
			setWPS(WPUtils.loadRoute(schedule.getRouteID(), config));			
	}
	
	public void tick()
	{
		double l = in_dist_route_end.read();
		
		int index = -1;
		for(int i = 0; i < length.length; i++)
			if(length[i] < l)
			{
				index = i;
				break;
			}
		
		if(index != -1)
		{
			out_curve_speed.write(speed[index]);
			out_length_to_curve.write(l-length[index]);
			
			if(index < length.length-1)
			{
				out_curve_speed2.write(speed[index+1]);
				out_length_to_curve2.write(l-length[index+1]);
			}
		}
		else
		{
			out_curve_speed.write(1*scale);
			out_length_to_curve.write(100000);
			
			out_curve_speed2.write(1*scale);
			out_length_to_curve2.write(100000);
		}
	}
	
	public static void main(String[] args) throws Exception
	{
		Sim s = new Sim();
		WayPoints wps = WPUtils.loadRoute("52.S", new StdConfig());
		RouteCurves r = new RouteCurves(null,null);
		r.setWPS(wps);
	}
}
