package team.app.models;

import sim.BooleanChannel;
import sim.DoubleChannel;
import sim.Entity;
import sim.CoordChannel;
import sim.StateChangeModel;
import team.app.wp.WPDriver;
import team.app.wp.WayPoints;
import team.gui.Coord;

public class RouteDriver extends Entity
{
	public DoubleChannel in_move_length = new DoubleChannel(this);
	
	public DoubleChannel out_length_to_end = new DoubleChannel(this);
	public CoordChannel out_position = new CoordChannel(this);
	public BooleanChannel out_finished = new BooleanChannel(this);
	public BooleanChannel out_event_finished;
	
	private WPDriver driver = new WPDriver();

	public RouteDriver()
	{
		regCommit(driver);		
		out_finished.initState(false);
		
		StateChangeModel tmp = new StateChangeModel();
		tmp.in.connect(out_finished);
		out_event_finished = tmp.out;
		out_event_finished.put(false);
	}
	
	public void setWayPoints(WayPoints wps)
	{
		driver.setWayPoints(wps);		
	}
	
	public void initToStart()
	{
		driver.reset();
		out_position.initState(driver.getPosition());
	}
	
	public boolean isFinished()
	{
		return out_finished.read();
	}
	
	public boolean eventFinished()
	{
		return out_event_finished.read();
	}
	
	public double getLengthTo(Coord coord)
	{
		return driver.getLengthTo(coord);
	}
	
	public double getLengthToEnd()
	{
		return driver.getLengthToEnd();
	}
	
	protected void read()
	{
		if(getSim().isDebugOn())
			System.out.println("Finished: " + isFinished());
	}

	protected void tick()
	{
		boolean val = driver.isFinsihed();
		if(val != out_finished.read())
			out_finished.put(val);
		
		driver.move(in_move_length.read());
		out_position.write(driver.getPosition());
		out_length_to_end.write(driver.getLengthToEnd());
	}
}
