package team.app.models;

import idb.dom.Node;

import java.util.List;

import sim.CoordChannel;
import sim.DoubleChannel;
import sim.Entity;
import team.app.IConfig;
import team.app.div.Random;
import team.app.view.StopPainter.IStops;
import team.app.wp.WPUtils;
import team.gui.Coord;

public class RouteStop extends Entity implements IStops
{
	public DoubleChannel in_time = new DoubleChannel(this);
	
	public CoordChannel out_next_stop = new CoordChannel(this);
	public DoubleChannel out_length_to_stop = new DoubleChannel(this);
	public DoubleChannel in_move_length = new DoubleChannel(this);
	
	private final BusSpeeder speeder;
	private final RouteDriver driver;
	private final Schedule schedule;
	private final IConfig config;
	
	private int hld_index = -1;
	private Node hld_node;
	private List hld_list;

	private boolean tmp_next_route;
	private double time = Double.MAX_VALUE;

	public RouteStop(IConfig app, RouteDriver driver, BusSpeeder speeder, Schedule schedule)
	{
		this.config = app;
		this.speeder = speeder;
		this.schedule = schedule;
		this.driver = driver;
		
		out_length_to_stop.initState(0);
	}
	
	public void setRouteID(String route_id)
	{
		hld_list = config.getConfig("rute.txt").selectAll("rute[id=\""+route_id +"\"]/hld");
		hld_index = -1;
		nextStop();
	}
	
	private void nextStop()
	{
		if(isFinished())
				return;
		
		hld_node = (Node) hld_list.get(++hld_index);
		out_length_to_stop.initState(driver.getLengthTo(getNextStopCoord()));
		
		//uLog.println(""+out_length_to_stop);
	}
	
	public boolean isFinished()
	{
		return !(hld_index < hld_list.size()-1);
	}
	
	public String getNextStopID()
	{
		return hld_node.nodeText("id");
	}
	
	public Coord getNextStopCoord()
	{
		if(hld_index == -1)
			return null;
		
		return WPUtils.parseWCoord(hld_node.nodeText("wcoord"));
	}
	
	public String getNextStopName()
	{
		return hld_node.nodeText("navn");
	}
	
	public double getNextStopLength()
	{
		return out_length_to_stop.read();
	}
	
	public double getNextStopTime()
	{
		double personer = 4;
		double variance = 1;
		
		String id = getNextStopID();
		Node n = config.getConfig("rute.txt").select("stats/hld[id='"+id+"']");
		if(n != null && n.nodeText("verdi") != null && n.nodeText("var") != null)
		{
			personer = Double.parseDouble(n.nodeText("verdi"));
			variance = Double.parseDouble(n.nodeText("var"));
		}
		
		double time = Random.randn2(personer*2.5, variance*2.5);
		if(time <= 0)
			time = 0;
		
		return time;
	}
	
	public boolean eventNextStop()
	{
		return out_length_to_stop.read() < 0;
	}

	
	protected void reset()
	{
		setRouteID(schedule.getRouteID());
	}
	
	protected void read()
	{
		tmp_next_route = false;
		
		if(schedule.eventNextRoute() && !schedule.isScheduleFinished())
			tmp_next_route = true;
	}
	
	protected void write()
	{
		if(tmp_next_route)
		{
			setRouteID(schedule.getRouteID());
		}
		
		if(!isFinished() && out_length_to_stop.read() < 1)
		{			
			if(hld_index == 0)
				speeder.setStopUntil(schedule.getRouteStartTime());
			else
			{
				speeder.setStopTime(getNextStopTime());
				time = getNextStopTime() + in_time.read(); 
			}
			
			nextStop();
		}
	}
	
	protected void tick()
	{
		if(in_time.read() > time)
		{
			//System.out.println("Passerer x, skal til: " + getNextStopName());
			out_next_stop.write(getNextStopCoord());
			time = Double.MAX_VALUE;		
		}
		
		out_length_to_stop.write(out_length_to_stop.getState() - in_move_length.read());		
	}
}
