package team.app.models;

import java.util.ArrayList;
import java.util.List;

import sim.Entity;
import team.app.IConfig;
import team.app.wp.WPUtils;
import team.app.wp.WayPoints;

public class Schedule extends Entity
{
	private List trip_list = new ArrayList();
	private List route_list = new ArrayList();
	private List start_list = new ArrayList();
	private int index = -1;
	
	private WayPoints wps = new WayPoints();
	
	private RouteDriver driver;
	private IConfig config;
	private boolean tmp_next_route;
	
	public Schedule(RouteDriver driver, IConfig config)
	{
		this.driver = driver;
		this.config = config;
	}
	
	public void loadTrip(String trip_id, String routeID, double time)
	{
		route_list.add(routeID);
		trip_list.add(trip_id);
		start_list.add(new Double(time));
	}
	
	public int[] getTripIDs()
	{
		int[] ids = new int[trip_list.size()];
		for(int i = 0; i < trip_list.size(); i++)
			ids[i] = Integer.parseInt((String) trip_list.get(i));
		
		return ids;
	}
	
	public void nextRoute()
	{		
		if(index >= route_list.size()-1)
			return;
		
		index++;
		
		String r_id = (String) route_list.get(index);
		wps = WPUtils.loadRoute(r_id, config);
		
		driver.setWayPoints(wps);
		//driver.setStopUntil(getRouteStartTime());
	}
	
	public double getRouteStartTime()
	{
		return ((Double)start_list.get(index)).doubleValue();
	}
	
	public double getNextRouteStartTime()
	{
		if(!(index+1 < start_list.size()))
			return -1;
		
		return ((Double)start_list.get(index+1)).doubleValue();
	}
	
	public String getRouteID()
	{
		return (String) route_list.get(index);
	}
	
	public String getScheduleID()
	{
		return (String) trip_list.get(index);
	}
	
	public int getTripID()
	{
		return Integer.parseInt((String) trip_list.get(index));
	}

	public boolean isScheduleFinished()
	{
		if(index < route_list.size()-1)
			return false;
		
		return true;
	}
	
	public boolean isRouteFinished()
	{
		return driver.isFinished();
	}
	
	public boolean eventNextRoute()
	{
		return driver.eventFinished();
	}
	
	protected void reset()
	{
		index = -1;
		nextRoute();
		driver.initToStart();
	}
	
	protected void read()
	{
		tmp_next_route = false;
		
		if(driver.isFinished())
			tmp_next_route = true;
	}
	
	protected void write()
	{
		if(tmp_next_route)
			nextRoute();
	}

	protected void tick()
	{
		
	}
}
