package team.app.models;

import sim.DoubleChannel;
import sim.Entity;
import sim.CoordChannel;
import team.app.IConfig;
import team.app.system.ScheduleCalc;
import team.app.system.loader.RouteStatLoader;
import team.app.system.loader.TableScheduleLoader;
import team.gui.Coord;

public class ScheduleDelayCalc extends Entity
{	
	public DoubleChannel in_time = new DoubleChannel(this);
	public CoordChannel in_pos = new CoordChannel(this);
	
	private Coord sch_pos = new Coord(0,0);
	private Coord est_pos = new Coord(0,0);
	
	private ScheduleCalc calc;
	private Schedule sch;
	private boolean tmp_next_route;
	
	private double sample_time = 0;

	public ScheduleDelayCalc(IConfig config, Schedule sch)
	{		
		this.sch = sch;
		calc = new ScheduleCalc(config, new TableScheduleLoader(config));
		//calc = new ScheduleCalc(config, new MeanScheduleLoader(config,"stat.txt"));
	}
	
	private void setRoute(String rid, int tripID, double sch_start_time)
	{
		calc.loadRoute(rid,tripID,sch_start_time);
	}
	
	public double getDelay()
	{
		return calc.getObsDelay();
	}
	
	public Coord getScheduledPosition()
	{
		return sch_pos;
	}
	
	public Coord getEstimatePosition()
	{
		return est_pos;
	}
	
	public double getScheduleLength()
	{
		return calc.getScheduleLength(in_time.read());
	}
	
	public void reset()
	{
		setRoute(sch.getRouteID(), sch.getTripID(), sch.getRouteStartTime());
	}
	
	public void read()
	{
		tmp_next_route = false;
		if(sch.eventNextRoute())
			tmp_next_route = true;
	}
	
	public void write()
	{
		if(tmp_next_route)
			reset();
	}
	
	public void post_tick()
	{
		if(in_time.read() > sample_time)
		{
			calc.regObservation(in_pos.read(), in_time.read());
			sample_time = in_time.read() + 60;
		}
		
		sch_pos = calc.getScheduledPosition(in_time.read());
		est_pos = calc.estimatePosition(in_time.read());
	}
}
