package team.app.system;

import java.util.ArrayList;
import java.util.regex.Pattern;

import nextgen.utils.uFile;
import team.gui.Coord;

public class DB implements IReg
{
	private StringBuilder text = new StringBuilder();
	private Register register;
	
	private double line_time = 0;
	private int line_index = 0;
	
	private String[] linjer;
	
	public DB(Register register)
	{
		this.register = register;
		register.registerListener(this);
	}

	public void regActiveSchedule(double time, int busID, int[] tripID)
	{		
		text.append("s,").append(format(time)).append(',');
		text.append(busID).append(',');
		for(int i = 0; i < tripID.length; i++)
			text.append(tripID[i]).append(',');
		
		text.append("\n");
	}
	
	public void regObs(double time, int busID, Coord pos, int techID)
	{		
		text.append("p,").append(format(time)).append(',');
		text.append(busID).append(',');
		text.append(format(pos.x)).append(',').append(format(pos.y)).append(',');
		text.append(techID).append(',');
		text.append("\n");
	}
	
	private String format(double d)
	{
		return Formatter.format(d);
	}
	
	public void save(String file)
	{
		try
		{
			uFile.writeString(text.toString(), file);
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public static String plainToRegxp(String plain)
	{
		return Pattern.compile(plain, Pattern.LITERAL).pattern();
	}
	
	public void load(String file)
	{
		try
		{
			String s = uFile.readString(file);
			text = new StringBuilder(s);
			linjer = text.toString().split(plainToRegxp("\n"));
			
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public double getTime()
	{
		return getTime(line_index);
	}
	
	private double getTime(int line_index)
	{
		return Double.parseDouble(linjer[line_index].split(plainToRegxp(","))[1]);
	}
	
	public void playTo(double toTime)
	{		
		for(; line_index < linjer.length-1; line_index++)
		{
			if(getTime(line_index)>toTime)
				break;
			
			playSingle();
		}
	}
	
	public void play(double time)
	{
		playTo(line_time + time);
	}
	
	public void playAll()
	{
		playTo(Double.MAX_VALUE);
	}
	
	public boolean playSingle()
	{
		if(!(line_index < linjer.length-1))
			return false;
			
		String[] ord = linjer[line_index].split(plainToRegxp(","));
		if(ord[0].equals("p"))
		{
			line_time = Double.parseDouble(ord[1]);
			register.regObs(line_time,
					Integer.parseInt(ord[2]), 
					new Coord(Double.parseDouble(ord[3]),Double.parseDouble(ord[4])), 
					Integer.parseInt(ord[5]));
		}
		else if(ord[0].equals("s"))
		{
			line_time = Double.parseDouble(ord[1]);
			int busID = Integer.parseInt(ord[2]);
			ArrayList trips = new ArrayList();
			
			for(int j = 3; j < ord.length && ord[j] != null; j++)
				trips.add(ord[j]);
			
			int[] tripID = new int[trips.size()];
			for(int j = 0; j < trips.size(); j++)
				tripID[j] = Integer.parseInt((String)trips.get(j));
				
			register.regActiveSchedule(line_time, busID, tripID);
		}
		else
			System.out.println("Hm: " + ord[0]);
		
		return true;
	}	
}
