package team.app.system;

import java.util.ArrayList;

import team.app.IConfig;
import team.app.StdConfig;
import team.app.delay.Plot;
import team.app.delay.Plot.Plotter;
import team.app.system.filter.IPosFilter;
import team.app.system.filter.TestFilter;
import team.app.system.loader.IScehduleLoader;
import team.app.system.loader.RouteStatLoader;
import team.app.system.loader.TableScheduleLoader;
import team.app.system.stat.History;
import team.app.system.stat.RouteStat;
import team.gui.Coord;

public class Register implements IReg
{	
	private ArrayList listeners = new ArrayList();
	
	public Register()
	{
		
	}
	
	public void registerListener(IReg listener)
	{
		listeners.add(listener);
	}
	
	public void regActiveSchedule(double time, int busID, int[] tripID)
	{
		for(int i = 0; i < listeners.size(); i++)
		{
			IReg list = (IReg) listeners.get(i);
			list.regActiveSchedule(time, busID, tripID);
		}
	}
	
	public void regObs(double time, int busID, Coord pos, int techID)
	{		
		for(int i = 0; i < listeners.size(); i++)
		{
			IReg list = (IReg) listeners.get(i);
			list.regObs(time, busID, pos, techID);
		}
	}
	
	//Test for MeanScheduleLoader
	public static void test_01()
	{
		Register reg = new Register();
		DB db = new DB(reg);
		db.load("log.txt");
		db.playTo(24*60*60*1);
		
		IConfig conf = new StdConfig();
		IScehduleLoader loader = new TableScheduleLoader(conf);
		//IScehduleLoader loader = new RouteStatLoader(conf,"stat.txt");
		
		ScheduleReg bus01 = new ScheduleReg(conf, loader,reg,0,1);
		ScheduleReg bus02 = new ScheduleReg(conf, loader,reg,0,2);
		
		IPosFilter filter = new TestFilter();
		//IPosFilter filter = new DiffAvgFilter(20);
		//IPosFilter filter = new AvgFilter(5);
		//IPosFilter filter = new LSFilter();
		//IPosFilter filter = new KalmanFilter();
		
		
		bus02.setFilter(filter);
		//delay11.setFilter(new MeanFilter());
		
		Plot p = new Plot("Posisjonering uten filter");
		//Plot p = new Plot("Posisjonering med filter (DIFF20)");
		//Plot p = new Plot("Posisjonering med filter (AVR5)");
		//Plot p = new Plot("Posisjonering med filter (AVR5)");
		//Plot p = new Plot("Posisjonering med filter (LSQ)");
		//Plot p = new Plot("Posisjonering med filter (Kalman)");
		
		Plotter d1 = p.getPlotter("GPS");
		Plotter d2 = p.getPlotter("GSM");
		Plotter m = p.getPlotter("Mean");
		
		db.playSingle();
		double time = db.getTime();
		double stime = time;
		
		RouteStat s = new RouteStat(new History("stat.txt"));
		//s.load("stat.txt");
		
		System.out.println(stime);
		
		while(time < stime + 25*60)
		{
			time += 2;
			db.playTo(time);
			
			
			double d1l = bus01.getCalc().estimateLength(time);
			if(d1l > 0)
			{
				//p.setX(d1l);
				d1.plot(d1l,time-stime);
			}
			double d2l = bus02.getCalc().estimateLength(time);
			if(d2l > 0)
			{
				//p.setX(d2l);			
				d2.plot(d2l,time-stime);
			}
			
			//System.out.println("time: " + (time-stime)/60);
			m.plot(s.estimateLength(time-stime, "52.S"),time-stime);
			
			//p.setX(d2l)
			
			/*p.setX(time);
			d1.plot(delay11.getCalc().estimateLength(time));
			d2.plot(delay2.getCalc().estimateLength(time));*/
			
			/*System.out.println(Formatter.format(delay2.getCalc().estimateDelay(time),6) + 
					" -> " + Formatter.format(delay2.getCalc().estimateLengthLeft(time),6)+
					"  (" +Formatter.format(delay11.getCalc().estimateDelay(time),6) + 
					" -> " + Formatter.format(delay11.getCalc().estimateLengthLeft(time),6)+")" +
					" = " + Formatter.format(delay2.getCalc().estimateDelay(time) - delay11.getCalc().estimateDelay(time), 6));*/
		}
		
		p.setVisible(true);
	}
	
	//Traindata for MeanScheduleLoader
	public static void test_02()
	{
		Register reg = new Register();
		DB db = new DB(reg);
		db.load("log.txt");
		//db.playTo(24*60*60*2);
		
		
		//Lage statestikk med GPS, hva med GSM?
		IConfig conf = new StdConfig();
		ScheduleReg delay11 = new ScheduleReg(conf, new TableScheduleLoader(conf),reg,0,1);
		ScheduleReg delay12 = new ScheduleReg(conf, new TableScheduleLoader(conf),reg,1,1);
		ScheduleReg delay13 = new ScheduleReg(conf, new TableScheduleLoader(conf),reg,2,1);
		
		//delay11.setFilter(filter);
		
		int mod = 4;
		delay11.setMod(mod);
		delay12.setMod(mod);
		delay13.setMod(mod);
		
		History stat = new History();
		delay11.regListener(stat);
		delay12.regListener(stat);
		delay13.regListener(stat);
		
		db.playAll();
		
		//stat.plot();
		//stat.plotMean("52.S");
		stat.save("stat.txt");
	}
	
	public static void main(String[] args) throws Exception
	{		
		test_01();
		//test_02();
	}
}
