package team.app.system;

import idb.dom.Node;

import java.util.ArrayList;

import sim.Time;
import team.app.IConfig;
import team.app.system.filter.IPosFilter;
import team.app.system.loader.IScehduleLoader;
import team.app.system.loader.TableScheduleLoader;
import team.gui.Coord;

public class ScheduleReg implements IReg
{
	private ScheduleCalc delay;
	private IConfig conf;
	private ArrayList stats = new ArrayList();
	
	private int[] trips;
	private int tindex = 0;
	private int bussID;
	private int techID;
	
	private int obs_day;
	
	private Node trip = Node.parse("test{id=-1}");
	private boolean finished;
	private IPosFilter filter;
	
	private int mod = 1;
	private int count = -1;
	
	public ScheduleReg(IConfig conf, Register reg, int busID, int techID)
	{
		this(conf,new TableScheduleLoader(conf),reg,busID,techID);
	}
	
	public void regListener(IEvent stat)
	{
		stats.add(stat);
	}
	
	public void setMod(int mod)
	{
		this.mod = mod;		
	}
	
	public void setFilter(IPosFilter filter)
	{
		this.filter = filter;		
	}
	
	public ScheduleReg(IConfig conf, IScehduleLoader loader, Register reg, int busID, int techID)
	{
		this.bussID = busID;
		this.conf = conf;
		this.techID = techID;
		
		reg.registerListener(this);
		delay = new ScheduleCalc(conf, loader);
	}
	
	public void regActiveSchedule(double time, int bussID, int[] tripID)
	{
		if(bussID != this.bussID)
			return;
		
		obs_day = (int)(time / (24*60*60));
		
		tindex = 0;
		trips = tripID;
		loadNextTrip();
	}
	
	public void regObs(double time, int bussID, Coord pos, int techID)
	{		
		if(this.bussID != bussID || this.techID != techID)
			return;
		
		if(!(count++ % mod == 0))
			return;
		
		if(delay.estimateLengthLeft(time) < 10)
			loadNextTrip();
		
		obs_day = (int)(time / (24*60*60));
		
		if(filter != null)
			pos = filter.filter(time, pos, this);
		
		delay.regObservation(pos, time);
		
		if(!finished)
		for(int i = 0; i < stats.size(); i++)
		{
			IEvent stat = (IEvent) stats.get(i);
			stat.regEvent(this);
		}
	}
	
	private void loadNextTrip()
	{
		if(tindex < trips.length)
		{
			System.out.println("trip: " + trips[tindex]);
			loadRouteID(trips[tindex++]);
			finished = false;
		}
		else
		{
			delay.loadRoute(null, 0, 0);
			finished = true;
			
			if(filter != null)
				filter.setRoute(null);
		}
	}
	
	private double getSchedule(int tripID)
	{
		if(!trip.nodeText("id").equals(String.valueOf(tripID)))
		{
			Node rute = conf.getConfig("rute.txt");
			trip = rute.select("trips/trip[id="+tripID+"]");
		}
		
		return Time.toDouble(trip.nodeText("tid"));
	}
	
	private String getRouteID(int tripID)
	{
		if(!trip.nodeText("id").equals(String.valueOf(tripID)))
		{
			Node rute = conf.getConfig("rute.txt");
			trip = rute.select("trips/trip[id="+tripID+"]");
		}
		return trip.nodeText("rid");
	}
	
	private void loadRouteID(int tripID)
	{
		String routeID = getRouteID(tripID);
		double route_sch = getSchedule(tripID) + obs_day*60*60*24;
		delay.loadRoute(routeID, tripID, route_sch);
		
		if(filter != null)
			filter.setRoute(routeID);
	}
	
	public boolean isIdle()
	{
		return finished;
	}
	
	public int getTripID()
	{
		if(finished)
			return -1;
		
		return trips[tindex-1];
	}
	
	public ScheduleCalc getCalc()
	{
		return delay;
	}
	
	
}
