package team.app.system.filter;

import org.jscience.mathematics.numbers.Number;
import org.jscience.mathematics.vectors.Vector;

import team.app.delay.Filter;
import team.app.system.ScheduleCalc;
import team.app.system.ScheduleReg;
import team.gui.Coord;

public class DiffAvgFilter implements IPosFilter
{
	int size = 0;
	double[] data = new double[10000];
	double[] ddata = new double[10000];
	double[] fddata;
	
	private final double fsize;
	
	public DiffAvgFilter(double fsize)
	{
		this.fsize = fsize;	
	}
	
	public Coord filter(double time, Coord pos, ScheduleReg reg)
	{
		ScheduleCalc c = reg.getCalc();		
		double messured_adherence = calcAdherence(time, pos, c);
		
		System.out.println(" ad: " + messured_adherence);
		
		double prev_m_adherence = 0;
		if(size > 0)
			prev_m_adherence = data[size-1];
		
		ddata[size] = messured_adherence - prev_m_adherence;
		data[size] = messured_adherence;
		size++;
		
		fddata = Filter.filter(Filter.createAvg(fsize), ddata, size);
		
		double filtered_adherence = 0;
		for(int i = 0; i < size; i++)
			filtered_adherence += fddata[i];
		
		return c.getScheduledPosition(time + filtered_adherence);
	}
	
	public static double calcAdherence(double obs_time, Coord obs_pos, ScheduleCalc c)
	{		
		Vector va = c.estimatePosition(obs_time).toVector();
		Vector vb = c.estimatePosition(obs_time+1).toVector();
		
		Vector vd = vb.minus(va);
		Vector vp = obs_pos.toVector().minus(va);
		
		double est_length = ((Number)vp.times(vd)).doubleValue();
		est_length /= Coord.length(vd);
		
		if(Double.isNaN(est_length))
			est_length = 0;
		
		est_length = est_length + c.estimateLength(obs_time);
		return c.getScheduleTime(est_length) - obs_time;
	}
	
	

	public double getFilterValue()
	{
		double filtered_adherence = 0;
		for(int i = 0; i < size; i++)
		{
			System.out.println(fddata[i] + ", ");
			filtered_adherence += fddata[i];
		}
		
		return filtered_adherence;
	}
	
	public void setRoute(String id)
	{
		size = 3;
		for(int i = 0; i < size; i++)
		{
			data[i] = 0;
			ddata[i] = 0;
		}
	}
}
