package team.app.system.loader;

import java.util.ArrayList;
import java.util.List;

import team.app.IConfig;
import team.app.StdConfig;
import team.app.system.stat.History;
import team.app.system.stat.RouteStat;
import team.app.wp.WPUtils;

public class RouteStatLoader implements IScehduleLoader
{
	private String stat_file;
	private IConfig conf;

	public RouteStatLoader(IConfig conf, String stat_file)
	{
		this.conf = conf;
		this.stat_file = stat_file;		
	}

	public List loadRouteData(String routeID, int tripID, double time)
	{
		List data_list = new ArrayList();
		RouteStat stat = new RouteStat(new History(stat_file));
		//stat.load(stat_file);
		
		double dx = 20;
		double length = WPUtils.getRouteLengthExEnds(routeID, conf);
		
		double x = 0;
		while(true)
		{
			if(x > length)
				x = length+0.001;
			
			ScheduleData data = new ScheduleData();
			data_list.add(data);
			
			data.acu_length = x;
			data.leg_length = dx;
			
			
			data.sch_time = stat.estimateTime(x/length, routeID);
			data.leg_time = data.sch_time - stat.estimateTime((x-dx)/length, routeID);
			data.leg_speed = data.leg_length / data.leg_time;
			
			if(x > length)
				break;
			
			x+= dx;
		}
		
		
		return data_list;
	}
	
	public static void main(String[] args) throws Exception
	{
		RouteStatLoader load = new RouteStatLoader(new StdConfig(),"stat.txt");
		List list = load.loadRouteData("52.S", 0, 0);
		for(int i = 0; i < list.size(); i++)
		{
			ScheduleData d = (ScheduleData) list.get(i);
			System.out.println(d);
			
		}
	}
}
