package team.app.system.loader;

import idb.dom.Node;

import java.util.ArrayList;
import java.util.List;

import team.app.IConfig;
import team.app.StdConfig;
import team.app.wp.WPDriver;
import team.app.wp.WPUtils;
import team.app.wp.WayPoints;

public class TableScheduleLoader implements IScehduleLoader
{
	private final double hld_time = 10;
	private final IConfig conf;

	public TableScheduleLoader(IConfig conf)
	{
		this.conf = conf;
	}
	
	public double getHldTime()
	{
		return hld_time;
	}
	
	public List loadRouteData(String routeID, int tripID, double time)
	{
		List data = new ArrayList();	
		WayPoints wps = WPUtils.loadRoute(routeID, conf);
		WPDriver driver = new WPDriver(wps);
		driver.reset();
		
		Node rute = conf.getConfig("rute.txt").select("rute[id='"+routeID+"']");
		List hlds = rute.selectAll("hld");
		
		Node hld = (Node) hlds.get(0);
		//driver.move(driver.getLengthTo(WPUtils.parseWCoord(hld.nodeText("wcoord"))));		
		
		double old_acc_length = 0;
		double old_time = 0;
		for(int i = 0; i < hlds.size(); i++)
		{
			hld = (Node) hlds.get(i);
			ScheduleData d = new ScheduleData();
			//d.name = hld.nodeText("navn");			
			
			d.sch_time = Double.parseDouble(hld.nodeText("tid"))*60;
			if(i != 0 && Math.abs(d.sch_time-old_time) < 0.001)
				d.sch_time += 0.01;
			
			d.leg_time = d.sch_time - old_time;
			//d.pos = WPUtils.parseWCoord(hld.nodeText("wcoord"));				
			d.leg_length = driver.getLengthTo(WPUtils.parseWCoord(hld.nodeText("wcoord")));
			d.acu_length = d.leg_length + old_acc_length;
			
			d.leg_speed = d.leg_length / (d.leg_time-hld_time);
			if(d.leg_speed < 0.1)
				d.leg_speed = 1;
			
			driver.move(d.leg_length);
			old_acc_length = d.acu_length;
			old_time = d.sch_time;
			
			//System.out.println(d);
			data.add(d);		
		}
		
		return data;
	}
	
	public static void main(String[] args) throws Exception
	{
		IConfig conf = new StdConfig();
		List list = new TableScheduleLoader(conf).loadRouteData("52.P", 0, 0);
		System.out.println(list.get(0));
		System.out.println(list.get(list.size()-1));
		System.out.println(WPUtils.getRouteLengthExEnds("52.P", conf));
	}
}
