package team.app.system.stat;

import java.util.ArrayList;
import java.util.HashMap;

import nextgen.utils.uFile;
import sim.Time;
import team.app.delay.Plot;
import team.app.delay.Plot.Plotter;
import team.app.system.ScheduleReg;
import team.app.system.DB;
import team.app.system.Formatter;
import team.app.system.IEvent;

public class History implements IEvent
{
	protected ArrayList data = new ArrayList();
	private HashMap trip_stat_map = new HashMap();
	
	public History()
	{
	}
	
	public History(String stat_file)
	{
		load(stat_file);
	}

	public void regEvent(Object source)
	{
		ScheduleReg l = (ScheduleReg) source;
		
		if(l.isIdle())
			return;
		
		TripData s = getStat(l);
		s.reg(l.getCalc().getObsLengthComplete(), l.getCalc().getObsRouteTime());		
	}
	
	public int size()
	{
		return data.size();
	}
	
	public TripData get(int index)
	{
		return (TripData) data.get(index);
	}
	
	/*public double estimateTime(double nlength, String ruteID)
	{
		double value = 0;
		double weights = 0;
		
		for(int i = 0; i < data.size(); i++)
		{
			TripData d = (TripData) data.get(i);
			if(!d.ruteID.equals(ruteID))
				continue;
			
			value += d.interpolateTime(d.length*nlength);
			weights += 1;
		}
		
		return value/weights;
	}
	
	public double estimateLength(double time, String ruteID)
	{
		double value = 0;
		double weights = 0;
		
		for(int i = 0; i < data.size(); i++)
		{
			TripData d = (TripData) data.get(i);
			if(!d.ruteID.equals(ruteID))
				continue;
			
			value += d.interpolateLength(time);
			weights += 1;
		}
		
		return value/weights;
	}*/
	
	private TripData getStat(ScheduleReg l)
	{
		Integer key = Integer.valueOf(l.getCalc().getObsDay()*100+l.getTripID());
		TripData d = (TripData) trip_stat_map.get(key);
		if(d == null)
		{
			d = new TripData();
			d.tripID = l.getTripID();
			d.weekDay = l.getCalc().getObsDay();
			d.ruteID = l.getCalc().getRouteID();
			d.time = l.getCalc().getScheduleStartTime();
			d.length = l.getCalc().getRouteLength();
			
			data.add(d);			
			trip_stat_map.put(key, d);
		}
		
		return d;
	}
	
	public void plotMean(String ruteID, Plot p)
	{
		Plotter m = p.getPlotter("mean");
		
		RouteStat stat = new RouteStat(this);
		for(double x = 0; x < 1; x+=0.001)
		{
			m.plot(x, stat.estimateTime(x, ruteID));
		}
		
		/*for(double x = 0; x < 40*60; x++)
		{
			m.plot(estimateLength(x, ruteID),x);
		}*/
	}
	
	public void plotMean(String ruteID)
	{
		Plot p = new Plot();	
		plotMean(ruteID, p);
		p.setVisible(true);
	}
	
	public void plotAll(String ruteID)
	{
		Plot p = new Plot();		
		
		for(int i = 0; i < data.size(); i++)
		{
			TripData d = (TripData) data.get(i);
			if(!d.ruteID.equals(ruteID))
				continue;
				
			plot(p, d);
		}
		
		p.setVisible(true);
	}
	
	public void plot(int[] trips)
	{
		Plot p = new Plot();		
		
		for(int i = 0; i < data.size(); i++)
		{
			TripData d = (TripData) data.get(i);
			
			boolean funnet = false;
			for(int j = 0; j < trips.length; j++)
				if(d.tripID == trips[j])
					funnet = true;
				
			if(!funnet)
				continue;
				
			plot(p, d);
		}
		
		p.setVisible(true);
	}
	
	public void plot(Plot p, TripData s)
	{
		Plotter t = p.getPlotter(String.valueOf(s.tripID));
		
		for(int i = 0; i < s.size; i++)
		{
			//p.setX(s.x[i]);
			t.plot(s.x[i], s.y[i]/60);
		}		
	}
	
	public void save(String file)
	{
		//trip, rute, dag, tidpdag -> data
		StringBuilder s = new StringBuilder();
		for(int i = 0; i < data.size(); i++)
		{
			TripData d = (TripData) data.get(i);
			s.append(d.tripID).append(',');
			s.append(d.ruteID).append(',');
			s.append(d.month).append(',');
			s.append(d.weekDay).append(',');
			s.append(Formatter.format(d.time)).append(',');
			s.append(Formatter.format(d.length)).append(',');
			s.append(d.size).append(',');			
			
			for(int j = 0; j < d.size; j++)
			{
				s.append(Formatter.format(d.x[j])).append(',');
				s.append(Formatter.format(d.y[j])).append(',');
			}
			s.append("\n");
		}
		
		try
		{
			uFile.writeString(s.toString(), file);
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public void load(String file)
	{
		try
		{
			String s = uFile.readString(file);
			String[] lines = s.toString().split(DB.plainToRegxp("\n"));
			for(int i = 0; i < lines.length; i++)
			{
				String[] ord = lines[i].split(DB.plainToRegxp(","));
				TripData d = new TripData();
				data.add(d);
				
				d.tripID = Integer.parseInt(ord[0]);
				d.ruteID = ord[1];
				d.month = Integer.parseInt(ord[2]);
				d.weekDay = Integer.parseInt(ord[3]);
				d.time = Double.parseDouble(ord[4]);
				d.length = Double.parseDouble(ord[5]);
				
				d.size = Integer.parseInt(ord[6]);
				
				for(int j = 0; j < d.size; j++)
				{
					d.x[j] = Double.parseDouble(ord[j*2+7]);
					d.y[j] = Double.parseDouble(ord[j*2+8]);
				}			
			}			
			
			
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		
	}
	
	public static void main(String[] args) throws Exception
	{
		History s = new History();
		s.load("stat.txt");
		s.plotMean("52.P");
		s.plotAll("52.S");
		s.plot(new int[] {1,5});
	}
}
