package team.app.system.stat;

import team.app.delay.Filter;
import team.app.delay.Plot;
import team.app.delay.Plot.Plotter;
import team.app.div.Random;


public class LSStat
{
	private double a = 0;
	private double b = 0;
	
	private int size = 0;
	private double[] y = new double[100000];
	private double[] x = new double[100000];
	private double[] fy = new double[100000];
	
	public void reg(double length, double time)
	{		
		y[size] = time;
		x[size] = length;
		size++;
	}
	
	public void clear()
	{
		size = 0;
	}
	
	public double estimateTime(double length)
	{
		return a + b*length;
	}
	
	public double estimateDTime(double lengtha, double lengthb)
	{
		return b*(lengthb-lengtha);
	}
	
	public void calcLeastSquaresMod()
	{
		//http://www.efunda.com/math/leastsquares/lstsqr1dcurve.cfm
		//http://www.weibull.com/LifeDataWeb/least_squares.htm
		//*http://www.ies.co.jp/math/java/misc/least_sq/least_sq.html
		
		int l_size = size;
		
		double sy = 0;
		double sxsq = 0;
		double sx = 0;
		double sxy = 0;		
		
		for(int i = 0; i < l_size; i++)
		{
			sy += y[i];
			sx += x[i];
			sxy += (x[i] * y[i]);
			sxsq += (x[i] * x[i]);			
		}
		
		if(l_size <= 3)
		{
			double maxx = 100000000;
			double meany = sy / l_size;
			
			sy += meany;
			sx += maxx;
			
			sxy += maxx * meany;
			sxsq += maxx * maxx;
			l_size ++;
		}
		
		b = (l_size*sxy - sx*sy)/(l_size * sxsq - sx*sx);
		
		if(Double.isNaN(b))
			b = 0;
		
		a = (sy - sx*b)/l_size;		
		//fy = Filter.filter(Filter.createAvg(3), y, size);
		//fy = Filter.myFilter(y, size);
	}
	
	public double getMSE()
	{
		//http://www.cnel.ufl.edu/nsbook/chapter13_Least_Squares.html
		
		double sesq = 0;
		for(int i = 0; i < size; i++)
		{
			double e = y[i] - (a + b*x[i]);
			sesq += e * e;
		}
		
		return sesq / (2 * size);
	}
	
	public double getME()
	{
		return Math.sqrt(getMSE());
	}
	
	public double getSplitPoint()
	{		
		double maxe = Double.MIN_VALUE;
		double maxp = 0;
		
		double minx = x[0];
		double maxx = x[size-1];
		double l = maxx - minx;
		
		for(int i = 0; i < size; i++)
		{
			if(x[i] < minx + l*0.2)
				continue;
			else if(x[i] > maxx - l*0.2)
				continue;
			
			double e = fy[i] - (a + b*x[i]);
			e *= e;
			
			if(e > maxe)
			{
				maxe = e;
				maxp = x[i];
			}
		}
		
		return maxp;
	}
	
	public void split(double bp)
	{
		LSStat s1 = new LSStat();
		LSStat s2 = new LSStat();
		
		for(int i = 0; i  < size; i++)
		{
			if(x[i] < bp)
				s1.reg(x[i], y[i]);
			else
				s2.reg(x[i], y[i]);
		}
		
		if(s1.size <= 3 || s2.size <= 3)
			return;
		
		s1.calcLeastSquaresMod();
		s2.calcLeastSquaresMod();
		
		System.out.println("s1 ME: " + s1.getME());
		System.out.println("s2 ME: " + s2.getME());
		
		s1.plot();
		s2.plot();
		
		if(s1.getME() > getME() / 2)
			s1.split(s1.getSplitPoint());
		else if(s2.getME() > getME() / 2)
			s2.split(s2.getSplitPoint());
		
	}
	
	public void plot()
	{
		Plot p = new Plot();
		//p.setPoint();
		
		Plotter t = p.getPlotter("delay");
		Plotter s = p.getPlotter("least square");
		Plotter f = p.getPlotter("filter avg(3)");
		
		for(int i = 0; i < size; i++)
		{
			p.setX(x[i]);
			t.plot(y[i]/60);
			s.plot(estimateTime(x[i])/60);
			f.plot(fy[i]/60);
		}
		p.setVisible(true);
	}
	
	public static void main(String[] args) throws Exception
	{
		Random.setTimeSeed();
		LSStat s = new LSStat();
		for(double x = 0; x < 10000; x+=500)
		{
			double delay = 1*60;
			if(x > 3000)
				delay = (x-3000)/1000*60 + 60;
			if(x > 6000)
				 delay = 4.5 * 60;
			
			double m_delay = Random.randn(delay,0.5*60);			
			s.reg(x,m_delay);
		}
		s.calcLeastSquaresMod();
		s.plot();
		System.out.println("ME: " + s.getME());
		System.out.println("BP: " + s.getSplitPoint());
		//s.split(s.getSplitPoint());
	}
}
