package team.app.view;

import java.awt.Graphics2D;
import java.awt.Image;

import sim.CoordChannel;
import team.app.App;
import team.gui.Coord;
import team.gui.IPaint;

public class ModelPainter implements IPaint
{
	private Image image = null;
	private double factor = 1;
	private boolean dynamic = false;
	private IPos data = null;
	
	private final App app;
	
	public ModelPainter(App app, String img_file)
	{
		this.app = app;
		app.shape_p.registerPainter(this);	
		
		image = app.loadImage("res/"+img_file);
	}
	
	public ModelPainter setSource(final CoordChannel c)
	{
		setSource(new IPos() {
			public Coord getPosition()
			{
				return c.read();
			}
		});
		
		return this;
	}
	
	public ModelPainter setSource(IPos pos)
	{
		data = pos;
		return this;
	}
	
	public ModelPainter setScale(double factor)
	{
		this.factor = factor;
		return this;
	}
	
	public ModelPainter setDynamicSize(boolean dynamic)
	{
		this.dynamic = dynamic;		
		return this;
	}

	public void paint(Graphics2D g2d)
	{
		if(data == null)
			return;
		
		Coord pos = data.getPosition();
		if(pos == null)
			return;
		
		if(!dynamic)
			app.shape_p.setNoScale(g2d, pos.x, pos.y);
		
		g2d.scale(factor, factor);
		g2d.translate(-image.getWidth(null)/2, -image.getHeight(null)/2);
		g2d.drawImage(image, 0, 0, null);		
	}
	
	public interface IPos
	{
		public Coord getPosition();
	}
}
