package team.app.wp;


import sim.ICommit;
import team.gui.Coord;

public class WPDriver implements ICommit
{
	private WayPoints wps;
	
	private Coord _pos = new Coord(0,0);
	private Coord pos = new Coord(0,0);
	
	private int _wp = 0;
	private int wp = 0;
	
	private boolean tmp_finished = true;
	private boolean _tmp_finished = true;
	
	
	public WPDriver(WayPoints wps)
	{
		setWayPoints(wps);
		reset();
	}
	
	public WPDriver()
	{
		
	}
	
	public void commit()
	{
		_pos.set(pos);
		_wp = wp;
		_tmp_finished = tmp_finished;
	}
	
	public void rollback()
	{
		pos.set(_pos);
		wp = _wp;
		tmp_finished = _tmp_finished;
	}
	
	public void setWayPoints(WayPoints wps)
	{
		this.wps = wps;
		reset(pos);
	}
	
	public WayPoints getWayPoints()
	{
		return wps;
	}
	
	public Coord getStartPosition()
	{
		return wps.getWP(0);
	}
	
	public Coord getPosition()
	{
		return pos;
	}
	
	public double getTotalLength()
	{
		double length = 0;
		if(wps.isEmpty())
			return length;
		
		Coord old = wps.getWP(0);
		for(int i = 1; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			length += wp1.distance(old);
			old = wp1;
		}
		
		return length;
	}
	
	public boolean isFinsihed()
	{		
		return tmp_finished;
	}
	
	public double getLengthToEnd()
	{
		double length = 0;
		if(wps.isEmpty())
			return length;
		
		Coord old = pos;
		for(int i = wp; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			length += wp1.distance(old);
			old = wp1;
		}
		
		return length;
	}
	
	
	public double getLengthToNextWP()
	{
		return pos.distance(wps.getWP(wp));
	}
	
	public double getLengthTo(Coord target)
	{
		Coord w0 = pos;
		
		double p_dist = 100000;
		double p_dist_acc = 0;
		
		double total_dist_acc = 0;
		
		for(int i = wp; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			Coord cp1 = WPUtils.getClosestPointToSegment(w0, wp1, target);
			if(cp1.distance(target) < p_dist)
			{
				p_dist = cp1.distance(target)-1;
				p_dist_acc = total_dist_acc + cp1.distance(w0);
			}
			
			total_dist_acc += wp1.distance(w0);
			w0 = wp1;
		}		
		
		return p_dist_acc;
	}	
	
	public void move(double meters)
	{		
		while(true)
		{
			if(wps.size() == 0 || meters < 0.01)
				return;
			
			Coord nwp = wps.getWP(wp);
			double dx = nwp.x - pos.x;
			double dy = nwp.y - pos.y;
			
			double l = Math.sqrt(dx*dx + dy*dy);
			
			if(meters < l)
			{
				pos.x += (dx / l)*meters;
				pos.y += (dy / l)*meters;				
				break;
			}
			else
			{
				meters -= l;
				if(wp < wps.size()-1)
					wp++;
				else
				{
					pos.set(nwp);
					tmp_finished = true;
					break;
				}
				
				pos.set(nwp);
			}
		}
	}
	
	public void reset()
	{
		reset(getStartPosition());
	}
	
	public void reset(Coord pos1)
	{
		pos.set(pos1);
		wp = 0;
		tmp_finished = false;
		commit();
	}
}
