package team.app.wp;

import idb.dom.Node;

import java.util.List;

import org.jscience.mathematics.numbers.Float64;
import org.jscience.mathematics.vectors.VectorFloat64;

import team.app.IConfig;
import team.gui.Coord;

public class WPUtils
{
	public static WayPoints loadRoute(String rid, IConfig conf)
	{
		WayPoints wps = new WayPoints();
		Node rute = conf.getConfig("rute.txt").select("rute[id=\""+rid+"\"]");
		List list = rute.node("lanes").nodes();
		for(int i = 0; i < list.size(); i++)
		{
			Node lane_id = (Node) list.get(i);
			String slane_id = lane_id.name();
			if(slane_id.charAt(0) == 'r')
				wps.addWPS(loadLaneReverse(slane_id.substring(1,slane_id.length()),conf));
			else			
				wps.addWPS(loadLane(lane_id.name(),conf));
		}
		return wps;
	}
	
	public static double getRouteLengthExEnds(String rid, IConfig conf)
	{
		/*Node rute = conf.getConfig("rute.txt").select("rute[id=\""+rid+"\"]");
		List hlds = rute.nodes("hld");
		Coord start = parseWCoord(((Node)hlds.get(0)).nodeText("wcoord"));
		Coord end = parseWCoord(((Node)hlds.get(hlds.size()-1)).nodeText("wcoord"));
		
		WayPoints wps = loadRoute(rid, conf);
		double length = WPDriver.getTotalLength(wps);
		length -= length - WPDriver.getDistance(wps, end, length);
		length -= WPDriver.getDistance(wps, start, 0);
		
		return length;*/
		
		return WPUtils.getTotalLength(loadRoute(rid, conf));
	}
	
	public static WayPoints loadLane(String lane_id, IConfig conf)
	{
		WayPoints wps = new WayPoints();
		Node lane = conf.getConfig("rute.txt").node("lanepoints").select("lane[id=\""+lane_id+"\"]");
		List list = lane.nodes("wcoord");
		for(int i = 0; i < list.size(); i++)
		{
			Node wc = (Node) list.get(i);
	    	wps.addWP(parseWCoord(wc.text()));
		}
		
		return wps;
	}
	
	public static WayPoints loadLaneReverse(String lane_id, IConfig conf)
	{
		WayPoints wps = loadLane(lane_id, conf);
		wps = wps.getReverse();		
		return wps;
	}
	
	public static Coord parseWCoord(String wcoord)
	{
		String c = wcoord;
		int index = c.indexOf(',');				    	
    	double x = Double.parseDouble(c.substring(0, index));
    	double y = Double.parseDouble(c.substring(index+1, c.length()));
    	return new Coord(x,y);
	}

	public static double getTotalLength(WayPoints wps)
	{
		double length = 0;
		if(wps.isEmpty())
			return length;
		
		Coord old = wps.getWP(0);
		for(int i = 1; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			length += wp1.distance(old);
			old = wp1;
		}
		
		return length;
	}

	public static Coord getPosition(WayPoints wps, double length)
	{
		Coord wp0 = wps.getWP(0);
		if(length <= 0)
			return wp0;
		
		for(int i = 1; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			double dist = wp1.distance(wp0);
			if(dist > length)
			{
				double dx = wp1.x - wp0.x;
				double dy = wp1.y - wp0.y;
				
				double l = Math.sqrt(dx*dx + dy*dy);
				Coord ret = new Coord(0,0);
				ret.set(wp0);
				ret.x += (dx / l)*length;
				ret.y += (dy / l)*length;
				return ret;
			}
			else
			{
				length -= dist;
				wp0 = wp1;
			}			
		}
		return wps.getWP(wps.size()-1);
	}

	public static double getDistance(WayPoints wps, Coord pos, double guess_length)
	{
		if(wps.isEmpty())
			return 0;
		
		Coord wp0 = wps.getWP(0);
		
		//closeness -> length
		double[] closeness = new double[wps.size()-1];
		double[] length = new double[wps.size()-1];
		
		double start_dist = 0;
		
		for(int i = 1; i < wps.size(); i++)
		{
			Coord wp1 = wps.getWP(i);
			Coord route_point = WPUtils.getClosestPointToSegment(wp0, wp1, pos);
			
			closeness[i-1] = route_point.distance(pos);
			length[i-1] = start_dist + wp0.distance(route_point);
			
			start_dist += wp1.distance(wp0);
			wp0 = wp1;
		}
		
		double error = 10000;
		int index = -1;
		
		for(int i = 0; i < closeness.length; i++)
		{
			if(length[i] < guess_length)
				continue;
			
			if(closeness[i] > error)
				break;
				
			error = closeness[i];			
			index = i;
		}
		
		for(int i = closeness.length-1; i >= 0; i--)
		{
			if(length[i] > guess_length)
				continue;
			
			
			if(closeness[i] > error)
				break;
			
			error = closeness[i];			
			index = i;
		}
		
		return length[index];
		
		
		
		/*double error = 1000000;
		int e_index = -1;
		for(int i = 0; i < closeness.length; i++)
		{			
			double t_error = closeness[i] + Math.abs(length[i]-guess_length) / 100;
			if(t_error < error)
			{
				error = t_error;
				e_index = i;
			}
		}
		
		return length[e_index];*/
	}

	public static Coord getClosestPointOnRoute(WayPoints wps, Coord pos)
	{
		if(wps.isEmpty())
			return null;
		
		Coord w0 = wps.getWP(0);
		
		Coord p = w0;
		double p_dist = 100000;		
		double total_dist_acc = 0;
		
		for(int i = 1; i < wps.size(); i++)
		{
			Coord w1 = wps.getWP(i);
			Coord cp = WPUtils.getClosestPointToSegment(w0, w1, pos);
			if(cp.distance(pos) < p_dist)
			{
				p_dist = cp.distance(pos);
				p = cp;
			}
			
			total_dist_acc += w1.distance(w0);
			w0 = w1;
		}		
		
		return p;
	}

	public static Coord getClosestPointToSegment(Coord a, Coord b, Coord pos)
	{
		//http://www.exsanity.freeola.com/nearestpoint.html
		//*http://softsurfer.com/Archive/algorithm_0102/algorithm_0102.htm
		//http://astronomy.swin.edu.au/~pbourke/geometry/pointline/source.c
	
		VectorFloat64 p1 = VectorFloat64.valueOf(new double[] {a.x,a.y});
		VectorFloat64 p2 = VectorFloat64.valueOf(new double[] {b.x,b.y});
		VectorFloat64 p = VectorFloat64.valueOf(new double[] {pos.x,pos.y});
		
		VectorFloat64 v = p2.minus(p1);
		VectorFloat64 w = p.minus(p1);
		
		Float64 c1 = w.times(v);
		if(c1.floatValue() <= 0)
			return a;
		
		Float64 c2 = v.times(v);
		if(c2.compareTo(c1) <= 0)
			return b;
		
		VectorFloat64 pb = p1.plus(v.times(c1.divide(c2)));
		return new Coord(pb);
	}
}
