package team.gui;

import org.jscience.mathematics.numbers.Float64;
import org.jscience.mathematics.structures.Field;
import org.jscience.mathematics.vectors.Matrix;
import org.jscience.mathematics.vectors.Vector;

/**
 * Konvertere lokale piksler til globale pixler
 * @author Fredrik Larsen
 *
 */
public class CoordTransform
{
	private Matrix trans_to_parent;
	private Matrix trans_to_world;
	private Matrix trans_to_local;
	private CoordTransform parent;
	
	public CoordTransform(double x1, double y1, double x2, double y2, double px1, double py1, double px2, double py2, CoordTransform parent)
	{
		this.parent = parent;
		set(x1, y1, x2, y2, px1, py1, px2, py2);
	}
	
	public CoordTransform(double sx, double sy, double ox, double oy, CoordTransform parent)
	{
		this.parent = parent;
		set(sx, sy, ox, oy);
	}
	
	public CoordTransform(Matrix transform)
	{
		trans_to_parent = transform;
		update();
	}
	
	public void set(double x1, double y1, double x2, double y2, double px1, double py1, double px2, double py2)
	{
		trans_to_parent = createTransformMatrix(x1, y1, x2, y2, px1, py1, px2, py2);
		update();
	}
	
	
	
	public void set(double sx, double sy, double ox, double oy)
	{
		trans_to_parent = createTransformMatrix(sx, sy, ox, oy);
		update();
	}
	
	public void setParent(CoordTransform parent)
	{
		this.parent = parent;
	}
	
	public void update()
	{
		trans_to_world = trans_to_parent;
		
		if(parent != null)
			trans_to_world = trans_to_world.times(parent.trans_to_world);
		
		trans_to_local = trans_to_world.inverse();
	}
	
	public Coord getParentCoord(double x, double y)
	{
		return new Coord(trans_to_parent.times(Vector.valueOf(new double[] {x,y,1})));
	}
	
	public Coord getWorldCoord(double x, double y)
	{
		return new Coord(trans_to_world.times(Vector.valueOf(new double[] {x,y,1})));
	}
	
	public Coord getLocalFromWorld(double wx, double wy)
	{
		return new Coord(trans_to_local.times(Vector.valueOf(new double[] {wx,wy,1})));
	}
	
	public Matrix getLocalMatrix()
	{
		return trans_to_parent;
	}
	
	public Matrix getWorldMatrix()
	{
		return trans_to_world;
		/*Matrix tmp = trans_to_parent;
		if(parent != null)
			tmp.times(parent.getWorldMatrix());
		return tmp;*/
	}
	
	public double getValue(int m, int n)
	{
		return ((Float64)trans_to_parent.get(m,n)).doubleValue();
	}
	
	public static Matrix createTransformMatrix(double fx1, double fy1, double fx2, double fy2, double tx1, double ty1, double tx2, double ty2)
	{		
		double sx = (tx2-tx1)/(fx2-fx1);
		double sy = (ty2-ty1)/(fy2-fy1);
		
		double ox = tx1-fx1*sx;
		double oy = ty1-fy1*sy;
		
		return createTransformMatrix(sx, sy, ox, oy);
	}
	
	public static Matrix createTransformMatrix(double sx, double sy, double ox, double oy)
	{
		Matrix m = Matrix.valueOf(new double[][] {
				{sx,0,ox},
				{0,sy,oy},
				{0,0,1}});
		
		return m;
	}
	
	public static Matrix createTransform(double l1x, double l1y,double l2x, double l2y,double l3x, double l3y,   double p1x, double p1y,double p2x, double p2y,double p3x, double p3y)
	{
		//http://testdrive.mapinfo.com/techsupp/miprod.nsf/kbase_by_product/0B57EA4620B5EBF285256A010082A16A
		//Finn matrise slik at l * M = p
		
		Matrix m = Matrix.valueOf(new double[][] {
				{l1x,l1y,1},
				{l2x,l2y,1},
				{l3x,l3y,1}});
		
		Matrix s = Matrix.valueOf(new double[][] {
				{p1x,p1y},
				{p2x,p2y},
				{p3x,p3y}});
		
		Matrix x = m.solve(s);
		x = x.transpose();
		
		Field[][] f = new Field[3][3];
		for(int i = 0; i < 2; i++)
		for(int j = 0; j < 3; j++)
				f[i][j] = x.get(i, j);
		
		f[2] = new Float64[] {Float64.ZERO,Float64.ZERO,Float64.valueOf(1)};
		x = Matrix.valueOf(f);		
		
		return x;
	}
	
	/*public static Matrix createTransform(double l1x, double l1y,double l2x, double l2y,double l3x, double l3y,   double p1x, double p1y,double p2x, double p2y,double p3x, double p3y)
	{
		//http://testdrive.mapinfo.com/techsupp/miprod.nsf/kbase_by_product/0B57EA4620B5EBF285256A010082A16A
		//Finn matrise slik at l * M = p
		
		Matrix m = Matrix.valueOf(new double[][] {
				{l1x,l1y,1},
				{l2x,l2y,1},
				{l3x,l3y,1}});
		
		Matrix s = Matrix.valueOf(new double[][] {
				{p1x,p1y,0},
				{p2x,p2y,0},
				{p3x,p3y,0}});
		
		Matrix x = m.solve(s);
		x = x.transpose();
		
		
		Field[][] f = new Field[3][3];
		for(int i = 0; i < 3; i++)
		for(int j = 0; j < 3; j++)
			f[i][j] = x.get(i, j);
		
		f[2][2] = Float64.valueOf(1);
		x = Matrix.valueOf(f);
		
		return x;
	}*/
	
	public static Matrix getUTM2WorldTransform()
	{
		//Trnd Stentralstatsjon, Nardokrysset, Glshaugen Nord
		//Matrix m = createTransform(569919.900, 7034926.420, 571282.652, 7031324.363, 570198.704, 7032968.805,     1991,-284, 3276,3324, 2233,1649);
		
		/*
		 * MolotTog: 1532,-351 -> 569439,7034980
		 * sttunveg: 1090,3742 -> 569075, 7030859
		 * Kirkvollen: 4253,2265 -> 572247, 7032412
		 */
		
		Matrix m = createTransform(569439,7034980, 569075, 7030859, 572247, 7032412,     1532,-351, 1090,3742, 4253,2265);
		return m;
	}
	
	public static void test1()
	{
		CoordTransform p = new CoordTransform(10,10,20,20,100,100,110,110,null);
		CoordTransform k = new CoordTransform(10,10,100,100,1,1,10,10,p);
		
		System.out.println(p.getParentCoord(10, 20));
		System.out.println(k.getWorldCoord(0,0));
	}
	
	public static void test2()
	{
		Matrix m = createTransform(0, 0, 0, 1, 1, 0, 5, 3, 3, 3, 5, 5);
		System.out.println(m);
		System.out.println(m.times(Matrix.valueOf(new double[][] {{1},{1},{1}})));
	}

	
	public static void main(String[] args) throws Exception
	{
		Matrix m = getUTM2WorldTransform();
		System.out.println(m);
		//System.out.println(m.times(Matrix.valueOf(new double[][] {{571335},{7031402},{1}})));
		//System.out.println(m.times(Matrix.valueOf(new double[][] {{567938},{7034584},{1}})));
	}
}
