package team.gui;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import org.jscience.mathematics.vectors.Matrix;


public class MapPainter implements IPaint
{
	private ArrayList images = new ArrayList();
	private ArrayList trans = new ArrayList();
	
	private final WorldPainter view;
	
	public MapPainter(WorldPainter view)
	{
		this.view = view;
		view.registerPainter(this);		
	}
	
	public BufferedImage getScaledInstance(Image img, int w, int h)
	{
		BufferedImage tmp = new BufferedImage(w,h,BufferedImage.TYPE_BYTE_INDEXED);
		tmp.getGraphics().drawImage(img, 0, 0, w, h, view);
		return tmp;		
	}
	
	public void paint(Graphics2D g2d)
	{
		Matrix iview = view.getInverseViewMatrix();
		
		for(int i = 0; i < images.size(); i++)
		{
			if(i > 0)
				g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			
			Image img = (Image) images.get(i);
			CoordTransform img_c = (CoordTransform) trans.get(i);
			
			Matrix itemMatrix = iview.times(img_c.getWorldMatrix());		
			CoordTransform transf = new CoordTransform(itemMatrix);
			Coord start = transf.getWorldCoord(0, 0);
			Coord end = transf.getWorldCoord(img.getWidth(view), img.getHeight(view));
			
			g2d.drawImage(img, (int)start.x, (int)start.y, (int)(end.x-start.x), (int)(end.y-start.y), view);
			//g2d.drawImage(getScaledInstance(img, (int)(end.x-start.x), (int)(end.y-start.y)), (int)start.x, (int)start.y, view);
		}
	}
	
	public void addImage(Image image, CoordTransform ctrans)
	{
		images.add(image);
		this.trans.add(ctrans);
	}
	
	public void addImage(String file, CoordTransform ctrans)
	{
		addImage(ImageComp.loadImage(file), ctrans);
	}
	
	public void addImage(Image image, int ox, int oy)
	{
		CoordTransform t = new CoordTransform(1, 1, -image.getWidth(view)/2+ox, -image.getHeight(view)/2+oy,view.getWorldTransform());
		addImage(image, t);
	}
	
	public void addImage(String file, int ox, int oy)
	{
		addImage(ImageComp.loadImage(file),ox,oy);
	}
}
