package team.gui;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import javax.swing.JPanel;


public class ShapePainter implements IPaint
{
	private ArrayList painters = new ArrayList();
	private final WorldPainter view;
	
	private AffineTransform org_transform;
	private AffineTransform session_transform;
	
	private double scale = 0;
	
	public ShapePainter(WorldPainter view)
	{
		this.view = view;
		view.registerPainter(this);
	}
	
	public void registerPainter(IPaint painter)
	{
		painters.add(painter);
	}
	
	private void transformAndScale(Graphics2D g2d)
	{
		CoordTransform tr = new CoordTransform(view.getInverseViewMatrix());
		Coord s = tr.getWorldCoord(0, 0);
		Coord e = tr.getWorldCoord(10, 10);
		
		g2d.translate(s.x, s.y);
		
		scale = (e.x-s.x)/10;
		g2d.scale((e.x-s.x)/10, (e.y-s.y)/10);		
	}
	
	public double getScale()
	{
		return scale;
	}
	
	public double[] getBoundryWCoords()
	{
		Coord pos1 = view.getWorldCoordFromWindowCoord(0, 0);
		Coord pos2 = view.getWorldCoordFromWindowCoord(view.getWidth(), view.getHeight());
			
		return new double[] {pos1.x,pos1.y,pos2.x,pos2.y};
	}
	
	public void setNoScale(Graphics2D g2d, double wcx, double wcy)
	{
		g2d.setTransform(session_transform);
		g2d.translate(wcx, wcy);
		double s = getScale();
		g2d.scale(1/s, 1/s);
	}
	
	public void reset(Graphics2D g2d)
	{
		g2d.setTransform(session_transform);
	}

	public void paint(Graphics2D g2d)
	{
		org_transform = g2d.getTransform();
		transformAndScale(g2d);
		session_transform = g2d.getTransform();
		
		for(int i = 0; i < painters.size(); i++)
		{
			IPaint p = (IPaint) painters.get(i);
			p.paint(g2d);
			
			g2d.setTransform(session_transform);
		}
		
		g2d.drawString("Dette er en test", 0, 0);
		g2d.setTransform(org_transform);
	}
	
	
	public static void test(final ShapePainter drawer, final JPanel view)
	{
		drawer.registerPainter(new IPaint() {
			public void paint(Graphics2D g2d)
			{
				drawer.setNoScale(g2d, 150, 150);
				g2d.drawRect(-20, -20, 40, 40);
			}});
		
		/*final Image buss = ImageComp.loadImage("res/buss.GIF");
		
		drawer.registerPainter(new IPaint() {
			public void paint(Graphics2D g2d)
			{
				drawer.setNoScale(g2d, 1000, 1000);
				//g2d.translate(1000, 1000);
				g2d.scale(0.5, 0.5);
				g2d.translate(-buss.getWidth(view)/2, -buss.getHeight(view)/2);
				g2d.drawImage(buss, 0, 0, view);
			}});*/
	}
}
