package team.gui;

import idb.dom.Node;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;

import nextgen.utils.uFile;

import org.jscience.mathematics.vectors.Matrix;

import team.app.div.HldDrawer;
import team.app.div.WCoordPainter;

public class WorldPainter extends JPanel
{
	private static final long serialVersionUID = -884563272680444281L;
	
	CoordTransform world = new CoordTransform(1,1,0,0,null);
	CoordTransform view = new CoordTransform(1,1,0,0,world);
	Matrix iview = view.getWorldMatrix().inverse();
	CoordTransform camera = new CoordTransform(5,5,1996,1467,view);
	
	ArrayList painters = new ArrayList();
	
	private int old_x = 0;
	private int old_y = 0;
	
	public WorldPainter()
	{		
		addMouseListener(new MouseAdapter(){
			
			public void mousePressed(MouseEvent e)
			{
				old_x = e.getX();
				old_y = e.getY();
			}

			public void mouseClicked(MouseEvent e)
			{
				Coord pos = getWorldCoordFromWindowCoord(e.getX(), e.getY());
				
				if(e.getButton() == 1)
				{
					camera.set(camera.getValue(0,0) , camera.getValue(1,1), pos.x, pos.y);
					repaint();
				}
			}
		});
		
		addMouseWheelListener(new MouseWheelListener() {

			public void mouseWheelMoved(MouseWheelEvent e)
			{
				double tmp = e.getWheelRotation();
				tmp = 1 + tmp*0.2;
				
				camera.set(camera.getValue(0,0)*tmp , camera.getValue(1,1)*tmp, camera.getValue(0,2), camera.getValue(1,2));
				repaint();
			}});
		
		addMouseMotionListener(new MouseMotionAdapter() {

			public void mouseDragged(MouseEvent e)
			{
				double x = e.getX();
				double y = e.getY();
				
				CoordTransform cw = new CoordTransform(camera.getWorldMatrix().inverse());			
				Coord end = cw.getLocalFromWorld(x, y);
				Coord start = cw.getLocalFromWorld(old_x, old_y);
				
				old_x = (int)x;
				old_y = (int)y;
				
				camera.set(camera.getValue(0,0) , camera.getValue(1,1), camera.getValue(0, 2)-(end.x-start.x), camera.getValue(1, 2)-(end.y-start.y));
				repaint();
			}
		});
	}
	
	public CoordTransform getWorldTransform()
	{
		return world;
	}
	
	public Coord getWorldCoordFromWindowCoord(int x, int y)
	{		
		return new CoordTransform(getInverseViewMatrix()).getLocalFromWorld(x, y);
	}
	
	public Matrix getInverseViewMatrix()
	{
		return iview;
	}
	
	
	public void registerPainter(IPaint painter)
	{
		painters.add(painter);
	}
	
	public void paint(Graphics g)
	{
		super.paint(g);		
		Rectangle clipBounds = getVisibleRect(); 
		view.set(1, 1, -clipBounds.width/2, -clipBounds.height/2);
		
		view.update();
		camera.update();
		
		iview = camera.getWorldMatrix();
		iview = iview.inverse();
		
		Graphics2D g2d = (Graphics2D) g;
		
		//g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		//g2d.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
		//g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		for(int i = 0; i < painters.size(); i++)
		{
			IPaint p = (IPaint) painters.get(i);
			p.paint(g2d);
		}
	}
	
	
	public static JFrame show(JPanel view)
	{
		JFrame frame = new JFrame("Dette er en test");
		frame.setSize(800, 600);
		frame.setLocation(300, 0);
		
		//frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		frame.add(view);
		frame.setVisible(true);
		return frame;
	}
	
	public static void test1() throws Exception
	{
		WorldPainter view = new WorldPainter();
		MapPainter map = new MapPainter(view);
		ShapePainter drawer = new ShapePainter(view);
		new HldDrawer(drawer);
		
		ShapePainter.test(drawer, view);		
		
		//map.addImage("300m.gif", 0,0);
		map.addImage("lib/300m.gif", new CoordTransform(246,324,1055,865,0,0,4053,2734,view.getWorldTransform()));
		//map.addImage("lib/kart.gif", new CoordTransform(782,426, 235,150, 7407,3983, 5,2,view.getWorldCoord()));
		//map.addImage("lib/kart 500.PNG", new CoordTransform(113.65,132.6, 812,604, 431,299, 3890,2648,view.getWorldCoord()));
		
		CoordTransform gps_trans = new CoordTransform(270670.0,7042238.7506812,   271721.0,7038404.86103542,  1991,-284,  3276,3324,view.getWorldTransform());
		
		
		System.out.println(gps_trans.getWorldCoord(270779.9575,7040283.0));

		WorldPainter.show(view);
	}
	
	
	public static void main(String[] args) throws Exception
	{
		if(args.length != 0 && args[0].equals("log"))
			System.setOut(new PrintStream(new File("log.txt")));
		
		Node maps = Node.parse(uFile.readString("arg.txt"));
		
		WorldPainter view = new WorldPainter();
		MapPainter map = new MapPainter(view);
		ShapePainter drawer = new ShapePainter(view);
		
		
		if(maps.nodeText("holdeplasser") != null && maps.nodeText("holdeplasser").equals("true"))
			new HldDrawer(drawer);
		
		//new HldPrinter(view);
		//new WCoordPrinter(" ",view);
		drawer.registerPainter(new WCoordPainter(" ",view));
		
		ShapePainter.test(drawer, view);
		
		/*Buss b1 = new Buss(drawer);
		b1.loadLane("1");
		b1.loadLane("2");
		b1.loadLane("3");
		b1.loadLane("4");
		b1.loadLaneReverse("4");
		b1.loadLane("5");
		b1.loadLaneReverse("5");
		b1.loadLaneReverse("3");
		b1.loadLaneReverse("2");
		b1.loadLaneReverse("1");
		
		b1.init(20, 0);*/

		
		List list = maps.nodes("map");
		for(int i = 0; i < list.size(); i++)
		{
			Node m = (Node) list.get(i);
			if(m.node("x") != null)
				continue;
			
			map.addImage(m.nodeText("img"), new CoordTransform(Double.parseDouble(m.nodeText("l1x")),
					Double.parseDouble(m.nodeText("l1y")),
					Double.parseDouble(m.nodeText("l2x")),
					Double.parseDouble(m.nodeText("l2y")),
					Double.parseDouble(m.nodeText("w1x")),
					Double.parseDouble(m.nodeText("w1y")),
					Double.parseDouble(m.nodeText("w2x")),
					Double.parseDouble(m.nodeText("w2y")),
					view.getWorldTransform()));
		}
		
		view.show(view);
		//new Animator(view).start();
	}
}
