package team.sim;

import java.util.ArrayList;
import java.util.List;

import team.sim.models.CarbonDating;
import team.sim.models.Logger;
import team.sim.models.Time;

public class World
{
	public static final int EVENT_INIT = Dispatcher.getEventID("EVENT_INIT");
	public static final int EVENT_RESET = Dispatcher.getEventID("EVENT_RESET");
	
	public static final int EVENT_SETUP = Dispatcher.getEventID("EVENT_SETUP");
	public static final int EVENT_READ = Dispatcher.getEventID("EVENT_READ");
	public static final int EVENT_WRITE = Dispatcher.getEventID("EVENT_WRITE");
	public static final int EVENT_READ_FINALE = Dispatcher.getEventID("EVENT_READ_FINALE");
	public static final int EVENT_TICK = Dispatcher.getEventID("EVENT_TICK");
	
	public static final int EVENT_DONE = Dispatcher.getEventID("EVENT_DONE");
	
	/**
	 * EVENT.WRITE og EVENT.TICK ikke ikke sls sammen fordi EVENT.WRITE
	 * benyttes til  kommunisere mellom agenter og EVENT.TICK benyttest til
	 *  oppdatere tilstand.
	 * 
	 * Dette for at man i tick skal f muligheten til  prosessere etter at
	 * alle funksjonskall til seg selv er utfrt.
	 * F.Eks i Stops.java: driver.setStopUntil i WRITE og utfres i TICK
	 * 
	 * Det er bare lov  lese tilstander fra andre agenter i EVENT.READ.
	 * Det er bare lov  kommunisere med andre agenter i EVENT.WRITE.
	 * Det er bare lov  oppdatere tilstand i EVENT.TICK
	 * 
	 */
	
	private int tcount = 0;
	private ArrayList list = new ArrayList();	
	public Dispatcher notifier = new Dispatcher(this);
	
	public boolean run = false;
	
	public World()
	{
		
	}
	
	public void run()
	{
		run = true;
		notifier.fireEvent(EVENT_RESET);
		while(run)
			tick();
		notifier.fireEvent(EVENT_DONE);
	}
	
	public void run(int ticks)
	{
		run = true;
		notifier.fireEvent(EVENT_RESET);
		for(int j = 0; j < ticks; j++)
		{
			if(!run)
				break;
			tick();
		}
		notifier.fireEvent(EVENT_DONE);
	}
	
	public void stop()
	{
		run = false;
	}
	
	public void tick()
	{
		tcount ++;
		notifier.fireEvent(EVENT_SETUP);
		notifier.fireEvent(EVENT_READ);
		notifier.fireEvent(EVENT_WRITE);
		notifier.fireEvent(EVENT_TICK);
	}
	
	public int getTickCount()
	{
		return tcount;
	}
	
	
	public void addEntity(IListener s)
	{
		list.add(s);
		notifier.addListener(s);
	}
	
	public void removeEntity(IListener s)
	{
		list.remove(s);
		notifier.removeListener(s);
	}
	
	public List getEntities()
	{
		return list;
	}
	
	public Object findInstanceOf(Class c)
	{
		for(int i = 0; i < list.size(); i++)
		{
			Object o = list.get(i);
			if(c.isInstance(o))
				return o;
		}
		return null;
	}	
	
	
	public static void main(String[] args) throws Exception
	{
		World sim = new World();
		sim.addEntity(new Time());
		sim.addEntity(new CarbonDating(sim));
		sim.addEntity(new Logger(sim));
		//sim.init();
		sim.run(500);
		//Time.getTime(sim).delta_time = 1;
		sim.run(100);
		//sim.destroy();
	}
}
